% Phase Variance, mod 2pi, First-Order Loop, 1st-order bandlimiting prefilter
%		plus statistics on skipped cycles.  A cycle is skipped when
%		absolute phase error exceeds 1 cycle.  At that point, the
%		nearest 1-cycle multiple becomes zero phase reference for skips.  
%   SINE NONLINEARITY
% Toolboxes Required: none
%[11,000 points/second, 90 seconds/million with Athlon 3000+ (sop = 2)]
clear;

% MODIFY PARAMETERS BELOW & GO
%*****************************
% Set either of the following 2 parameters and set the other to zero.
NS = 1; % N/S in Loop Bandwidth
No = 0; % Noise Power Density, per Hz, to Signal Ratio
Sources = 2; % 1 or 2 independent noise sources
FoverL = 1; % (Filter Bn)/(Loop Bn) =< SmpRate/K, = for effectively no filter
Phinit = 0;  % radian initial phase
K = 1;      % rad/sec unity-gain frequency
SmpRate = 100*K; % sample rate
OutInc = 1/K;% period between outputs
ncmpo = 1000;  % computed output points.
Offset = 0.2;  % = 0 for correct variance values, offset at PD output (same sign as steady-state output)
truncatePh = 1; % 1:truncate phase (must for statistics); 0:do not.
		% Choose Graphical Output
	%  sop=1:  Compute array
	%  sop=2:  Compute running stats (no array)
	%  sop=3:  Compute array and running stats
sop = 2;  % Set sop = 2 normally.
strtplt = 1; % skip first plotted points in statistical plots up to strtplt
%******************************
%compute effective (net) noise bandwidth in Hz:
if FoverL == 1
    Bn = K/8;
else
    Bn = (K/4)* (1/(1-FoverL^-2) + FoverL/(1-FoverL^2));
end
fprintf('\n\nPHASE STATISTICS (mod 2 pi), Bandlimited Noise   NB1(4/13/07)')
fprintf('\n1st-Order Loop with SINUSOIDAL Phase Detector Characteristic & 1st-Order Noise Filter')
fprintf('\nK = %g rad/sec = closed-loop BW = open-loop unity-gain frequency.  Offset = %g.',K,Offset)
fprintf('\nLoop Noise Bandwidth (linear): %g Hz, Filter Noise Bandwidth: %g Hz, ratio = %g',K/4,FoverL*K/4,FoverL)
fprintf('\nNet Noise Bandwidth (linear) Bno = %g Hz.  Number of noise sources: %u\n',Bn,Sources)
% Compute multiplier of noise distribution having unity power.
if NS == 0,
	Dn = No; NS = Bn*Dn; 
	fprintf('Noise Density to Signal Ratio = %g /Hz.',No)
else
	Dn = NS/Bn;
 	fprintf('Signal/Noise in Bno = %g.',1/NS)
end
Kn = sqrt(Dn*SmpRate/2); % noise multiplier to give variance equivalent to density Dn over half of the sampling rate 
Nfilt = SmpRate/K/FoverL; % (sampling rate)*(filter time constant = 1/Wo)

last = 1 + ncmpo; % index of last output
ending = OutInc*ncmpo; % End time
SmpPerOut = SmpRate*OutInc; % samples per output period
t = linspace(0,ending,last); % linear sequence of times for plots
dt = (t(2) - t(1))/SmpPerOut; % sample time increment

if NS < 0.03 % less than 2% error from this approximation or from 14 terms
    sig2 = NS;
else
    sig2 = (pi^2)/3;
    for kk = 1:14
        sig2 = sig2 +4*((-1)^kk/kk^2)*besseli(kk,1/NS)/besseli(0,1/NS);
    end
end % if
fprintf('  Variance in Bno is So2 = %g rad^2',NS)
fprintf('\nTheoretical nonlinear variance, 1st order loop, same So2 in Bno: %g rad^2\n',sig2)

% To help judge inaccuracy of ignoring sampling,
%   compute extra phase shift produced at unity gain
SampPhShift = -(90/pi)*K*OutInc/SmpPerOut; % radians lag from sampling at WL = K
fprintf('Added phase at unity O.L. gain due to sampling = %g deg.\n',SampPhShift)
if truncatePh
	fprintf('Phase will be shown in range 0.5 cycle, modulo 1 cycle.\n')	
else
	fprintf('Phase range will be unrestricted.\n')
end

%    BEGIN SIMULATION
y = Phinit; % initial phase at output
NL = sin(-Phinit); % <<<<<<<<<<<< THE NON-LINEARITY
e = -Phinit; % initial phase error
freq(1) = K*NL; % initial frequency at output
Nplus = 0; Nminus = 0; % skipped cycles
Fsum(1) = 0; Fnet(1) = 0; % rate of skipping

if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
	if e>0
		e = rem(e+pi,2*pi) - pi;
	else
		e = rem(e-pi,2*pi) + pi;
	end
end % if truncatePh
ersum = 0; ersum2 = 0; 
msv(1) = NS; % start plot at linear value (avoid problem with ratio in dB)
if (sop~= 2) 
	er(1) = e;
end
Noise1 = 0; Noise2 = 0; NoiseUn1 = 0; NoiseUn2 = 0;% filtered and unfiltered noise voltages
vari1 = 0; vari2 = 0; varo1 = 0; varo2 = 0; varo = 0; % noise variances at filter inputs and outputs plus net
running = last; i = 1; 
while running
    if (running > 10) % initialize arrays for improved processing speed
        Fnet(last)=0;
        Fsum(last)=0;
        PDnoise(last)=0;
        freq(last)=0;
        mnn(last)=0;
        t(last)=0;
        tshrt(last)=0;
        msv(last)=0;
        varshrt(last)=0;
    end % preassigning arrays
	while i < last, % compute loop responses
		i = i + 1;
		for j = 1:SmpPerOut, % loop through unrecorded samples
            k = (i-2)*SmpPerOut + j + 1;
			NL = sin(-y); % <<<<<<<<<<<< THE NON-LINEARITY 
            NoiseUn1 = randn; NoiseUn2 = randn; %noise at filter inputs
            vari1 = (NoiseUn1^2 + vari1*(k-1))/k;
            vari2 = (NoiseUn2^2 + vari2*(k-1))/k;
            Noise1 = Noise1*(1-1/Nfilt)+NoiseUn1/Nfilt;
            Noise2 = Noise2*(1-1/Nfilt)+NoiseUn2/Nfilt;
            varo1 = (Noise1^2 + varo1*(k-1))/k;
            varo2 = (Noise2^2 + varo2*(k-1))/k;
            if Sources == 2,              
                PDn = Kn*(Noise1*sin(y)+Noise2*cos(y));
            else
                PDn = Kn*Noise1;
            end
            varo = (PDn^2 + varo*(k-1))/k;
			u1 = NL + Offset + PDn; % initial input to sampler = constant Hold output -> same at beginning and end of dt
			y = y + u1*K*dt;
            % Count cycle skips
            eu = -y/(2*pi);
			if eu - Nplus + Nminus > 1
				Nplus = fix(eu + Nminus);
			elseif eu - Nplus + Nminus < -1
				Nminus = -fix(eu - Nplus);
			end
		end % for j			
        PDnoise(i) = PDn;
		e = -y; % untruncated error
		if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
			if e>0
				e = rem(e+pi,2*pi) - pi;
			else
				e = rem(e-pi,2*pi) + pi;
			end
		end % if truncatePh
		if (sop ~= 2)
			er(i) = e;
		end
		if (sop == 2)||(sop == 3)
			ersum = ersum + e;
			ersum2 = ersum2 + e^2;
			msv(i) = ersum2/(i);
			mnn(i) = ersum/(i);
			Fsum(i) = (Nplus + Nminus)/(dt*SmpPerOut*(i-1));
			Fnet(i) = (Nplus - Nminus)/(dt*SmpPerOut*(i-1));
		end
		% corresponding frequency out from Eq. (6.43) with d3 = 0
		freq(i) = K*u1; 
	end % for i
	
	% END SIMULATION.  The rest is plotting.
	
    clf;
    ncmpo = last - 1;
    ending = ncmpo*OutInc;
    t = linspace(0,ending,last); % reestablish time vector
    
    %****** Start Statistical Plots	
    tshrt = t(strtplt:last); varshrt = msv(strtplt:last);
    subplot(221);
    plot([t(strtplt) t(last)],[NS NS],'g',[t(strtplt) t(last)],[sig2 sig2],'m',tshrt,varshrt,'r')
    title('mean square phase (rad^2)')
    grid
    legend('linear','simple (FP)','nonlinear',0)
    subplot(223);
    plot(tshrt,10*log10(varshrt/NS),'b')
    title('rms phase ratio (dB)')
    grid
    subplot(222);
    plot(tshrt,mnn(strtplt:last),'k')
    title('mean phase error (rad)')
    grid
    subplot(224);
    plot(tshrt,Fnet(strtplt:last),'g',tshrt,Fsum(strtplt:last),'r')
    title('skips/sec, total and net')
    grid
    xlabel('seconds')
    %end plotting
    
    if (sop == 1)||(sop == 3) % compute from array
        arvar = std(er)^2;
        logarvar = 10*log10(arvar/NS);
        armnn = mean(er);
        fprintf('\nEstimate from error array:')
        fprintf('\n  output variance %g rad^2\n  ratio to So2 %g dB',arvar,logarvar);
        fprintf('\n  mean phase error %g rad\n  %g points including initial phase\n',armnn,last);
    end % for error array stats
    
    if (sop == 2)||(sop == 3) % compute from running stats
        if Sources == 1
            fprintf('\n  Source 1 variance: %g rad^2',Kn^2*vari1)
            fprintf('\n  Filtered variance: %g rad^2',varo)
        else
            fprintf('\n  Source 1 variance: %g,   Source 2 variance: %g rad^2',Kn^2*vari1, Kn^2*vari2)
            fprintf('\n  Filtered var. 1: %g, Filtered var. 2: %g, Total: %g rad^2',Kn^2*varo1, Kn^2*varo2, varo)
        end
        variance = (msv(last)-(mnn(last))^2)*last/(last-1);
        fprintf('\n  Mean square phase error is variance if mean is known to be zero:')
        fprintf('\n    mean square phase %g rad^2, phase variance %g rad^2',msv(last),variance);
        fprintf('\n  %g skipped cycles with %g net skips',Nplus+Nminus,Nplus-Nminus);
        fprintf('\n  mean phase error %g rad\n  %g points (%g seconds) including initial phase\n',mnn(last),last,last*OutInc);
    end % for running stats
    
    sound(sin(6283*[0:1/8192:0.2]),8192); %0.2 seconds of 1 kHz tone;
    running = input('    Enter additional points (0 to quit, <0 for key in). ');
    if (running < 0)
        running = -running;
        fprintf('\n    Keyboard entry.  Type "return" to resume program.\n')
        keyboard
    end
    last = last + running;
end % while running
%Notes:
%Mean square phase plot: "linear" and "simple (FP)" (Fokker-Plank) are for 0 offset.
%Plotted ratio is (mean square phase out)/(variance in linear loop).