% Phase Variance, mod 2pi, First-Order Loop, Butterworth prefilter,
%		plus statistics on skipped cycles.  A cycle is skipped when
%		absolute phase error exceeds 1 cycle.  At that point, the
%		nearest 1-cycle multiple becomes zero phase reference for skips.  
%   SINE NONLINEARITY
% Toolboxes Required: signal
%[57 min./1M, 34 seconds/10K with Athlon 3000+ (sop = 2)]
clear;
for Limiting = [1] % variable 1
for NS = [.01 .04] % variable 2
for FoverL3 = [10]*pi/2  % variable 3 = (filter video bw)/(loop 3 dB bw) 
%                           = FoverLn*pi/2, where FoverLn is ratio to loop noise bw
clear Fnet Fsum freq mnn t tshrt msv varshrt 
fid = fopen('runrec.txt','a');

% MODIFY PARAMETERS BELOW & GO
%*****************************
%Limiting = 0; % 0: OFF,  1: ON.  Signal + Noise hard limited.
FN = 2; %Normalize noise in [1 = Filter RF BW] or [2 = Net video noise BW] (N/S in Filter Video Bandwidth = PhaseNoise/S in RF BW)
% Set either of the following parameters and set the other to zero.
%NS = .5; % N/S in normalizing BW
No = 0; % Noise Power Density, per Hz, to Signal Ratio
Sources = 2; % 1 or 2 independent noise sources (not applicable with limiting)
%FoverL3 = 1.6666; % (Filter Video Bn)/(Loop -3 dB BW)
K = 1;      % rad/sec unity-gain frequency
SmpRate = 100*K; % sample rate
OutInc = 1/K;% period between outputs
ncmpo = 5000;  % computed output points.
Offset = 0;  % = 0 for correct variance values, offset at PD output (same sign as steady-state output)
Phinit = 0;  % radian initial phase
truncatePh = 1; % 1:truncate phase (must for statistics); 0:do not.
FL3 = K/2/pi; FLn = K/4; Ff = FoverL3*FL3; % Loop -3dB and noise BWs and filter video BW in Hz.
FvNet = FL3*atan(Ff/FL3); % Net video noise bandwidth
if FN == 1
    Fnorm = 2*Ff;
else
    Fnorm = FvNet;
end
%Filter
 forder = 4; % order and dB ripple of Butterworth filter
 fco = 1.95*Ff/SmpRate; % fco*(SmpRate/2) = (BnFilter).  BnFilter correction factor 0.97ww5.
 if fco < .01
     SRd = ceil(.001/fco); %Sampling rate divisor to prevent problems due to excessive rate/BW ratio
     fco = fco*SRd;
 else
     SRd = 1;
 end
 [bf af] = butter(forder,fco); % Butterworth filter coefficients
 [y,zf1] = filter(bf, af, randn(1,forder)); % initial filter states
 [y,zf2] = filter(bf, af, randn(1,forder));
		% Choose Graphical Output
strtplt = 1; % skip first plotted points in statistical plots up to strtplt
noplot = 0; % 1 to prevent plotting
	%  sop=1:  Compute array
	%  sop=2:  Compute running stats (no array)
	%  sop=3:  Compute array and running stats
sop = 2;  % Set sop = 2 normally.
if noplot
    sop = 2;
end;
%******************************
fprintf('\n\nPHASE STATISTICS (mod 2 pi), Noise through Butterworth filter, iterated: NB1bwi(4/13/07)');
fprintf('\n1st-Order Loop with SINUSOIDAL Phase Detector Characteristic & Rectangular Noise Filter');
fprintf('\nK = %g rad/sec = closed-loop BW = open-loop unity-gain frequency.  Offset = %g.',K,Offset);
fprintf('\nPreFilter (%g samples/sec) Video Noise Bandwidth, Bvf = %g Hz.',SmpRate/SRd,Ff);
fprintf('\nLoop (%g samples/sec.) Linear: Noise BW, B0nL = %g Hz, -3dB BW, B03L = %g Hz.',SmpRate,K/4,FL3);
fprintf('\nNet Noise Bandwidth (video), FvNet = %g Hz. Ratios: Bvf/B03L = %g, BvF/B0nL = %g.',FvNet,FoverL3,4*Ff/K);  
fprintf(fid,'\n\nPHASE STATISTICS (mod 2 pi), Noise through rectangular filter, iterated: NB1bwi(11/27/04)');
fprintf(fid,'\n1st-Order Loop with SINUSOIDAL Phase Detector Characteristic & Rectangular Noise Filter');
fprintf(fid,'\nK = %g rad/sec = closed-loop BW = open-loop unity-gain frequency.  Offset = %g.',K,Offset);
fprintf(fid,'\nPreFilter (%g samples/sec) Video Noise Bandwidth, Bvf = %g Hz.',SmpRate/SRd,Ff);
fprintf(fid,'\nLoop (%g samples/sec.) Linear: Noise BW, B0nL = %g Hz, -3dB BW, B03L = %g Hz.',SmpRate,K/4,FL3);
fprintf(fid,'\nNet Noise Bandwidth (video), FvNet = %g Hz. Ratios: Bvf/B03L = %g, BvF/B0nL = %g.',FvNet,FoverL3,4*Ff/K);  
if Limiting
    fprintf('\n  Hard limiting.\n');
    fprintf(fid,'\n  Hard limiting.\n');
    Sources = 2;
else
    fprintf('\nNumber of noise sources: %u.  No limiting.\n',Sources);
    fprintf(fid,'\nNumber of noise sources: %u.  No limiting.\n',Sources);
end

% Compute multiplier of noise distribution having unity power.
if NS == 0,
	Dn = No; NS = Fnorm*Dn; 
	fprintf('Noise Density to Signal Ratio = %g /Hz (L = S_phi)',No);
    fprintf(fid,'Noise Density to Signal Ratio = %g /Hz (L = S_phi)',No);
else
	Dn = NS/Fnorm;
 	fprintf('S/N in Filter = %g, S/PN = %g, S/N in FvNet = %g',Fnorm/Ff/2/NS,Fnorm/Ff/NS,Fnorm/FvNet/NS);
 	fprintf(fid,'S/N in Filter = %g, S/PN = %g, S/N in FvNet = %g',Fnorm/Ff/2/NS,Fnorm/Ff/NS,Fnorm/FvNet/NS);
end
Kn = sqrt(Dn*SmpRate/SRd/2); % noise multiplier to give variance equivalent to density Dn over half of the sampling rate 
last = 1 + ncmpo; % index of last output
ending = OutInc*ncmpo; % End time
SmpPerOut = SmpRate*OutInc; % samples per output period
t = linspace(0,ending,last); % linear sequence of times for plots
dt = (t(2) - t(1))/SmpPerOut; % sample time increment

if NS < 0.03 % less than 2% error from this approximation or from 14 terms
    sig2 = NS;
else
    sig2 = (pi^2)/3;
    for kk = 1:14
        sig2 = sig2 +4*((-1)^kk/kk^2)*besseli(kk,1/NS)/besseli(0,1/NS);
    end % for
end % if
%fprintf('  FLn = Loop Noise BW.  Variance in FLn is So2 = %g rad^2',NS)
%fprintf('\nTheoretical nonlinear variance, 1st order loop, same So2 in FLn, no filter: %g rad^2\n',sig2)

% To help judge inaccuracy of ignoring sampling,
%   compute extra phase shift produced at unity gain
SampPhShift = -(90/pi)*K*OutInc/SmpPerOut; % radians lag from sampling at WL = K
fprintf('\nAdded phase at unity O.L. gain due to sampling = %g deg.\n',SampPhShift);
fprintf(fid,'\nAdded phase at unity O.L. gain due to sampling = %g deg.\n',SampPhShift);
if truncatePh
	fprintf('Phase will be shown in range 0.5 cycle, modulo 1 cycle.\n');	
	fprintf(fid,'Phase will be shown in range 0.5 cycle, modulo 1 cycle.\n');	
else
	fprintf('Phase range will be unrestricted.\n');
	fprintf(fid,'Phase range will be unrestricted.\n');
end

%    BEGIN SIMULATION
y = Phinit; % initial phase at output
NL = sin(-Phinit); % <<<<<<<<<<<< THE NON-LINEARITY
e = -Phinit; % initial phase error
freq(1) = K*NL; % initial frequency at output
Nplus = 0; Nminus = 0; % skipped cycles
Fsum(1) = 0; Fnet(1) = 0; % rate of skipping

if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
	if e>0
		e = rem(e+pi,2*pi) - pi;
	else
		e = rem(e-pi,2*pi) + pi;
	end
end % if truncatePh
ersum = 0; ersum2 = 0; 
msv(1) = NS; % start plot at linear value (avoid problem with ratio in dB)
if (sop~= 2) 
	er(1) = e;
end
vari1 = 0; vari2 = 0; varo1 = 0; varo2 = 0; varo = 0; k2 = 0; % noise variances at filter inputs and outputs plus net
running = last; i = 1; 
while running
    if (running > 10)
        t(last)=0;
        if ~noplot
            Fnet(last)=0;
            Fsum(last)=0;
            freq(last)=0;
            mnn(last)=0;
            tshrt(last)=0;
            msv(last)=0;
            varshrt(last)=0;
        end
    end % preassigning arrays
	while i < last, % compute loop responses
		i = i + 1;
		for j = 1:SmpPerOut, % loop through unrecorded samples
            k1 = (i-2)*SmpPerOut + j + 1;
            if rem(k1-1.9999,SRd)<0.0002 % sample noise at loop rate / SRd
                k2 = k2 + 1;
                NoiseUn1 = randn; NoiseUn2 = randn; %noise at filter input
                vari1 = (NoiseUn1^2 + vari1*(k2-1))/k2;
                vari2 = (NoiseUn2^2 + vari2*(k2-1))/k2;
                [Noise1, zf1] = filter(bf,af,NoiseUn1,zf1); % noise and filter state from unfiltered noise and previous state
                [Noise2, zf2] = filter(bf,af,NoiseUn2,zf2);
                varo1 = (Noise1^2 + varo1*(k2-1))/k2;
                varo2 = (Noise2^2 + varo2*(k2-1))/k2;
                if Limiting
                    DelP = atan2(Noise1,Noise2+1/Kn);
                    PDn = 0;
                    varo = (DelP^2 + varo*(k2-1))/k2;
                else
                    if Sources == 2,              
                        PDn = Kn*(Noise1*sin(y)+Noise2*cos(y));
                    else
                        PDn = Kn*Noise1;
                    end
                    varo = (PDn^2 + varo*(k2-1))/k2;
                    DelP = 0;
                end
            end % if noise sample
			NL = sin(DelP-y); % <<<<<<<<<<<< THE NON-LINEARITY 
            u1 = NL + Offset + PDn; % initial input to sampler = constant Hold output -> same at beginning and end of dt
			y = y + u1*K*dt;
            % Count cycle skips
            eu = -y/(2*pi);
			if eu - Nplus + Nminus > 1
				Nplus = fix(eu + Nminus);
			elseif eu - Nplus + Nminus < -1
				Nminus = -fix(eu - Nplus);
			end
		end % for j			
		e = -y; % untruncated error
		if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
			if e>0
				e = rem(e+pi,2*pi) - pi;
			else
				e = rem(e-pi,2*pi) + pi;
			end
		end % if truncatePh
		if (sop ~= 2)
			er(i) = e;
		end
		if (sop == 2)||(sop == 3)
			ersum = ersum + e;
			ersum2 = ersum2 + e^2;
            if ~noplot
                msv(i) = ersum2/(i);
                mnn(i) = ersum/(i);
                Fsum(i) = (Nplus + Nminus)/(dt*SmpPerOut*(i-1));
                Fnet(i) = (Nplus - Nminus)/(dt*SmpPerOut*(i-1));
            end
		end
		% corresponding frequency out from Eq. (6.43) with d3 = 0
		if ~noplot
            freq(i) = K*u1; 
        end
	end % for i
	
	% END SIMULATION.  The rest is plotting.
    
    clf;
    ncmpo = last - 1;
    ending = ncmpo*OutInc;
    if ~noplot
        t = linspace(0,ending,last); % reestablish time vector
    end
    
    %****** Start Statistical Plots	
    if ~noplot
        tshrt = t(strtplt:last); varshrt = msv(strtplt:last);
        subplot(221);
        plot([t(strtplt) t(last)],[NS NS],'g',[t(strtplt) t(last)],[sig2 sig2],'m',tshrt,varshrt,'r')
        title('mean square phase (rad^2)')
        grid
        legend('linear','simple (FP)','nonlinear',0)
        subplot(223);
        plot(tshrt,10*log10(varshrt/NS),'b')
        title('rms phase ratio (dB)')
        grid
        subplot(222);
        plot(tshrt,mnn(strtplt:last),'k')
        title('mean phase error (rad)')
        grid
        subplot(224);
        plot(tshrt,Fnet(strtplt:last),'g',tshrt,Fsum(strtplt:last),'r')
        title('skips/sec, total and net')
        grid
        xlabel('seconds')
    end
    % end plotting
    
    if (sop == 1)||(sop == 3) % compute from array
		arvar = std(er)^2;
		logarvar = 10*log10(arvar/NS);
		armnn = mean(er);
		fprintf('\nEstimate from error array:')
        fprintf('\n  output variance %g rad^2\n  ratio to So2 %g dB',arvar,logarvar);
		fprintf('\n  mean phase error %g rad\n  %g points including initial phase\n',armnn,last);
	end % for error array stats
    
    if (sop == 2)||(sop == 3) % compute from running stats
        logvar = 10*log10(ersum/i/NS);
        if Sources == 1
            fprintf('\n  Source 1 variance: %g rad^2',Kn^2*vari1);
            fprintf('\n  Filtered variance: %g rad^2',varo);
            fprintf(fid,'\n  Source 1 variance: %g rad^2',Kn^2*vari1);
            fprintf(fid,'\n  Filtered variance: %g rad^2',varo);
        else
            fprintf('\n  Source 1 variance: %g,   Source 2 variance: %g rad^2',Kn^2*vari1, Kn^2*vari2);
            fprintf('\n  Filtered var. 1: %g, Filtered var. 2: %g, Total: %g rad^2',Kn^2*varo1, Kn^2*varo2, varo);
            fprintf(fid,'\n  Source 1 variance: %g,   Source 2 variance: %g rad^2',Kn^2*vari1, Kn^2*vari2);
            fprintf(fid,'\n  Filtered var. 1: %g, Filtered var. 2: %g, Total: %g rad^2',Kn^2*varo1, Kn^2*varo2, varo);
        end        
        Nskip = Nplus+Nminus; Dur = last*OutInc;
        fprintf('\n  (Filt1 out var.)/Specified = %g should be near 1.',Kn^2*varo1/NS*Fnorm/Ff);
        fprintf(fid,'\n  (Filt1 out var.)/Specified = %g should be near 1.',Kn^2*varo1/NS*Fnorm/Ff);
        if ~Limiting
            fprintf('\n    (Note: these rad units are equivalents using linear Kp.)');
            fprintf(fid,'\n    (Note: these rad units are equivalents using linear Kp.)');
        end
        variance = (ersum2 - ersum^2/i)/(i-1);
        fprintf('\n  Mean square phase error is variance if mean is known to be zero:');
        fprintf('\n    mean square phase %g rad^2, phase variance %g rad^2',ersum2/i,variance);
        fprintf('\n  mean phase error %g rad.  %g skipped cycles with %g net skips.',ersum/i,Nskip,Nplus-Nminus);
        fprintf('\n  Skip rate, F: F/(Filter RF BW) = %g.  F/(Net video noise BW) = %g.',Nskip/Dur/2/Ff,Nskip/Dur/FvNet);
        fprintf('\n  %g points (%g seconds) including initial phase\n', last, Dur);        
        fprintf(fid,'\n  Mean square phase error is variance if mean is known to be zero:');
        fprintf(fid,'\n    mean square phase %g rad^2, phase variance %g rad^2',ersum2/i,variance);
        fprintf(fid,'\n  mean phase error %g rad.  %g skipped cycles with %g net skips.',ersum/i,Nskip,Nplus-Nminus);
        fprintf(fid,'\n  Skip rate, F: F/(Filter RF BW) = %g.  F/(Net video noise BW) = %g.',Nskip/Dur/2/Ff,Nskip/Dur/FvNet);
        fprintf(fid,'\n  %g points (%g seconds) including initial phase\n', last, Dur);        
    end % for running stats
    
    sound(sin(6283*[0:1/8192:0.2]),8192); %0.2 seconds of 1 kHz tone;
    running = 0;
    pause(1);
end % while running
fclose(fid);
end % variable 3
end % variable 2
end % variable 1
%Notes:
%Mean square phase plot: "linear" and "simple (FP)" (Fokker-Plank) are for 0 offset.
%Plotted ratio is (mean square phase out)/(variance in linear loop).