% Phase Variance, mod 2pi, First-Order Loop, rectangular prefilter,
%		plus statistics on skipped cycles.  A cycle is skipped when
%		absolute phase error exceeds 1 cycle.  At that point, the
%		nearest 1-cycle multiple becomes zero phase reference for skips.  
%   SINE NONLINEARITY, 1st order data hold after filter
% [1st-order hold worked at 0.001=r where 0-order failed even at 0.01=r,
% same filter sampling rates (tf = 1)]
%  Toolboxes Required: signal
%[1370 points/second, 73 seconds/100K with Athlon 3000+ (sop = 2)]
clear;

% MODIFY PARAMETERS BELOW & GO
%*****************************
% Set either of the following parameters and set the other to zero.
test =0; tf = 1; %test = 1 for filter Smp Rate = loop SR; tf = divisor for filter SR (1 normal)
FN = 2; %Normalize noise in [1 = Filter video BW] or [2 = Net video noise BW] (N/S in Filter Video Bandwidth = PhaseNoise/S in RF BW)
NS = .5; % N/S in normalizing BW
No = 0; % Noise Power Density, per Hz, to Signal Ratio
Sources = 2; % 1 or 2 independent noise sources
FoverL3 = 10*pi/2; % (Filter Video Bn)/(Loop -3 dB BW)
Phinit = 0;  % radian initial phase
K = 1;      % rad/sec unity-gain frequency
SmpRate = 400*K; % sample rate
OutInc = 1/K;% period between outputs
FL3 = K/2/pi; FLn = K/4; Ff = FoverL3*FL3; % Loop -3dB and noise BWs and filter video BW in Hz.
FvNet = FL3*atan(Ff/FL3); % Net video noise bandwidth
if FN == 1
    Fnorm = Ff;
else
    Fnorm = FvNet;
end
%Filter
 forder = 4; fripple = 0.5; % order and dB ripple of Chebychef filter
 fco = 1.834*FoverL3*FL3/SmpRate; % fco*(SmpRate/2) = (BnFilter) = FoverL3*BL3.  BnFilter correction factor 0.917.
 if fco < .01 && ~test
     SRd = ceil(.01*tf/fco); %Sampling rate divisor to prevent problems due to excessive rate/BW ratio
     fco = fco*SRd;
 else
     SRd = 1;
 end
 [bf af] = cheby1(forder,fripple,fco); % Butterworth filter coefficients
 [y,zf1] = filter(bf, af, randn(1,forder)); % initial filter states
 [y,zf2] = filter(bf, af, randn(1,forder));
ncmpo = 1000;  % computed output points.  Student Matlab  1023(v3.5) or 8191(v4) or 16383(v5)
Offset = 0; % = 0 for correct variance values, offset at PD output (same sign as steady-state output)
truncatePh = 1; % 1:truncate phase (must for statistics); 0:do not.
		% Choose Graphical Output
strtplt = 1; % skip first plotted points in statistical plots up to strtplt
noplot = 0; % 1 to prevent plotting
	%  sop=1:  Compute array
	%  sop=2:  Compute running stats (no array)
	%  sop=3:  Compute array and running stats
sop = 2;  % Set sop = 2 normally.
if noplot
    sop = 2;
end;
%******************************
fprintf('\n\nPHASE STATISTICS (mod 2 pi), Noise through rectangular filter: NB1ch1(4/13/07)')
fprintf('\n1st-Order Loop with SINUSOIDAL Phase Detector Characteristic')
fprintf('\n  & 4th-order Chebychev Noise Filter with First-Order Data hold')
fprintf('\nK = %g rad/sec = closed-loop BW = open-loop unity-gain frequency.  Offset = %g.',K,Offset)
fprintf('\nFilter Video Noise Bandwidth: %g Hz, Loop -3 dB Bandwidth (linear): %g Hz, ratio = %g',Ff,FL3,FoverL3)
fprintf('\nLoop: %g samples/sec.  Filter: %g samples/sec.',SmpRate,SmpRate/SRd)
fprintf('\nLoop Noise Bandwidth (linear) Bno = %g Hz -> Net Noise Bandwidth (video) = %g Hz',K/4,FvNet)  
fprintf('\nNumber of noise sources: %u\n',Sources)
% Compute multiplier of noise distribution having unity power.
if NS == 0,
	Dn = No; NS = Fnorm*Dn; 
	fprintf('Noise Density to Signal Ratio = %g /Hz (L = S_phi)',No)
else
	Dn = NS/Fnorm;
 	fprintf('S/N in Filter = %g, S/PN = %g, S/N in net = %g',Fnorm/Ff/2/NS,Fnorm/Ff/NS,Fnorm/FvNet/NS);
end
Kn = sqrt(Dn*SmpRate/SRd/2); % noise multiplier to give variance equivalent to density Dn over half of the sampling rate 
last = 1 + ncmpo; % index of last output
ending = OutInc*ncmpo; % End time
SmpPerOut = SmpRate*OutInc; % samples per output period
t = linspace(0,ending,last); % linear sequence of times for plots
dt = (t(2) - t(1))/SmpPerOut; % sample time increment

if NS < 0.03 % less than 2% error from this approximation or from 14 terms
    sig2 = NS;
else
    sig2 = (pi^2)/3;
    for kk = 1:14
        sig2 = sig2 +4*((-1)^kk/kk^2)*besseli(kk,1/NS)/besseli(0,1/NS);
    end
end % if
%fprintf('  FLn = Loop Noise BW.  Variance in FLn is So2 = %g rad^2',NS)
%fprintf('\nTheoretical nonlinear variance, 1st order loop, same So2 in FLn, no filter: %g rad^2\n',sig2)

% To help judge inaccuracy of ignoring sampling,
%   compute extra phase shift produced at unity gain
SampPhShift = -(90/pi)*K*OutInc/SmpPerOut; % radians lag from sampling at WL = K
fprintf('\nAdded phase at unity O.L. gain due to sampling = %g deg.\n',SampPhShift)
if truncatePh
	fprintf('Phase will be shown in range 0.5 cycle, modulo 1 cycle.\n')	
else
	fprintf('Phase range will be unrestricted.\n')
end

%    BEGIN SIMULATION
y = Phinit; % initial phase at output
NL = sin(-Phinit); % <<<<<<<<<<<< THE NON-LINEARITY
e = -Phinit; % initial phase error
freq(1) = K*NL; % initial frequency at output
Nplus = 0; Nminus = 0; % skipped cycles
Fsum(1) = 0; FvNet(1) = 0; % rate of skipping

if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
	if e>0
		e = rem(e+pi,2*pi) - pi;
	else
		e = rem(e-pi,2*pi) + pi;
	end
end % if truncatePh
ersum = 0; ersum2 = 0; 
var(1) = NS; % start plot at linear value (avoid problem with ratio in dB)
if (sop~= 2) 
	er(1) = e;
end
Noise1 = 0; Noise2 = 0; % filtered noise voltages
vari1 = 0; vari2 = 0; varo1 = 0; varo2 = 0; varo = 0; k2 = 0; % noise variances at filter inputs and outputs plus net
running = last; i = 1; 
while running
    if (running > 10) && ~noplot
        FvNet(last)=0;
        Fsum(last)=0;
        PDnoise(last)=0;
        freq(last)=0;
        mnn(last)=0;
        t(last)=0;
        tshrt(last)=0;
        var(last)=0;
        varshrt(last)=0;
    end % preassigning arrays
	while i < last, % compute loop responses
		i = i + 1;
		for j = 1:SmpPerOut, % loop through unrecorded samples
            k1 = (i-2)*SmpPerOut + j + 1;
			NL = sin(-y); % <<<<<<<<<<<< THE NON-LINEARITY 
            if rem(k1-1.9999,SRd)<0.0002 % sample noise at loop rade / SRd
                k2 = k2 + 1; k3 = k1;
                NoiseUn1 = randn; NoiseUn2 = randn; %noise at filter input
                vari1 = (NoiseUn1^2 + vari1*(k2-1))/k2;
                vari2 = (NoiseUn2^2 + vari2*(k2-1))/k2;
                Noise1was = Noise1; Noise2was = Noise2;
                [Noise1, zf1] = filter(bf,af,NoiseUn1,zf1); % noise and filter state from unfiltered noise and previous state
                [Noise2, zf2] = filter(bf,af,NoiseUn2,zf2); 
                varo1 = (Noise1^2 + varo1*(k2-1))/k2;
                varo2 = (Noise2^2 + varo2*(k2-1))/k2;
            end % if noise sample
            Noise1held = Noise1 + (Noise1 - Noise1was)*(k1 - k3)/SRd;
            Noise2held = Noise2 + (Noise2 - Noise2was)*(k1 - k3)/SRd;
            if Sources == 2,              
                PDn = Kn*(Noise1held*sin(y)+Noise2held*cos(y));
            else
                PDn = Kn*Noise1held;
            end
            varo = (PDn^2 + varo*(k2-1))/k2;
            
			u1 = NL + Offset + PDn; % initial input to sampler = constant Hold output -> same at beginning and end of dt
			y = y + u1*K*dt;
            % Count cycle skips
            eu = -y/(2*pi);
			if eu - Nplus + Nminus > 1
				Nplus = fix(eu + Nminus);
			elseif eu - Nplus + Nminus < -1
				Nminus = -fix(eu - Nplus);
			end
		end % for j			
        if ~noplot
            PDnoise(i) = PDn;
        end
		e = -y; % untruncated error
		if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
			if e>0
				e = rem(e+pi,2*pi) - pi;
			else
				e = rem(e-pi,2*pi) + pi;
			end
		end % if truncatePh
		if (sop ~= 2)
			er(i) = e;
		end
		if (sop == 2)||(sop == 3)
			ersum = ersum + e;
			ersum2 = ersum2 + e^2;
            if ~noplot
                var(i) = ersum2/(i);
                mnn(i) = ersum/(i);
                Fsum(i) = (Nplus + Nminus)/(dt*SmpPerOut*(i-1));
                FvNet(i) = (Nplus - Nminus)/(dt*SmpPerOut*(i-1));
            end
		end
		% corresponding frequency out from Eq. (6.43) with d3 = 0
		if ~noplot
            freq(i) = K*u1; 
        end
	end % for i
	
	% END SIMULATION.  The rest is plotting.
	
	clf;
    ncmpo = last - 1;
    ending = ncmpo*OutInc;
    if ~noplot
        t = linspace(0,ending,last); % reestablish time vector
    end
    %****** Start Statistical Plots	
    if ~noplot
        tshrt = t(strtplt:last); varshrt = var(strtplt:last);
        subplot(221);
        plot([t(strtplt) t(last)],[NS NS],'g',[t(strtplt) t(last)],[sig2 sig2],'m',tshrt,varshrt,'r')
        title('phase variance (rad^2)')
        grid
        legend('linear','theoretical','nonlinear',4)
        subplot(223);
        plot(tshrt,10*log10(varshrt/NS),'b')
        title('variance ratio (dB)')
        grid
        subplot(222);
        plot(tshrt,mnn(strtplt:last),'k')
        title('mean phase error (rad)')
        grid
        subplot(224);
        plot(tshrt,FvNet(strtplt:last),'g',tshrt,Fsum(strtplt:last),'r')
        title('skip frequency (skips/sec), total and net')
        grid
        xlabel('seconds')
    end
    % end plotting
	
	if (sop == 1)||(sop == 3) % compute from array
		arvar = std(er)^2;
		logarvar = 10*log10(arvar/NS);
		armnn = mean(er);
		fprintf('\nEstimate from error array:')
        fprintf('\n  output variance %g rad^2\n  ratio to So2 %g dB',arvar,logarvar);
		fprintf('\n  mean phase error %g rad\n  %g points including initial phase\n',armnn,last);
	end % for error array stats
    
    if (sop == 2)||(sop == 3) % compute from running stats
        logvar = 10*log10(ersum/i/NS);
        fprintf('\nRunning statistics for known zero mean:')
        if Sources == 1
            fprintf('\n  Source 1 variance: %g rad^2',Kn^2*vari1)
            fprintf('\n  Filtered variance: %g rad^2',varo)
        else
            fprintf('\n  Source 1 variance: %g,   Source 2 variance: %g rad^2',Kn^2*vari1, Kn^2*vari2)
            fprintf('\n  Filtered var. 1: %g, Filtered var. 2: %g, Total: %g rad^2',Kn^2*varo1, Kn^2*varo2, varo)
        end
        fprintf('\n  (Filt1 out var.)/Specified = %g should be near 1.',Kn^2*varo1/NS*Fnorm/Ff);
        fprintf('\n  output variance %g rad^2\n  ratio to So2 %g dB',ersum2/i,logvar);
        Nskip = Nplus+Nminus; Dur = last*OutInc;
        fprintf('\n  %g skipped cycles with %g net skips.  F/(2*FiltVideoBW) = %g',Nskip,Nplus-Nminus,Nskip/Dur/2/Ff);
        fprintf('\n  mean phase error %g rad\n  %g points (%g seconds) including initial phase\n',ersum/i, last, Dur);
    end % for running stats
    
    sound(sin(6283*[0:1/8192:0.2]),8192); %0.2 seconds of 1 kHz tone;
    running = input('    Enter additional points (0 to quit, <0 for key in). ');
    if (running < 0)
        running = -running;
        fprintf('\n    Keyboard entry.  Type "return" to resume program.\n')
        keyboard
    end
	last = last + running;
end % while running
