% Phase Variance, mod 2pi, Second-Order Loop, 1st-order bandlimiting prefilter
%		plus statistics on skipped cycles.  A cycle is skipped when
%		absolute phase error exceeds 1 cycle.  At that point, the
%		nearest 1-cycle multiple becomes zero phase reference for skips.
%  SINE NONLINEARITY
%  Toolbox Required: control
% [22,640 points/minute, 26.5 seconds/10,000 points (at 100 samples per output) with Athlon 3000+]
clear;

% MODIFY PARAMETERS BELOW & GO
%*****************************
% Set either of the following parameters and set the other to zero.
NS = .5; % N/S in Loop Bandwidth
No = 0; % Noise Power Density, per Hz, to Signal Ratio
Sources = 2; % 1 or 2 independent noise sources
FoverL = 1; % set to 0 for no filter.
    %(Filter Bn)/(Loop Bn) =< SmpRate/(4*(Loop Bn))
Phinit = 0;  % radian initial phase
Winit = 0; % radian/sec initial frequency
Wn = 1;      % rad/sec natural frequency
z = .707;    % damping factor
alpha = 0;   % Eq. (6.5)
SmpPerWL = 100; % ratio of sample rate to unity-gain radian bandwidth 
TshowXWL = 1; % (period between outputs) X (unity-gain radian bandwidth)
ncmpo = 2000;  % computed output points.  Student Matlab  1023(v3.5) or 8191(v4) or 16383(v5)
Offset = 0; % offset at PD output (same sign as steady-state output)
truncatePh = 1; % 1:truncate phase; 0:do not
		% Choose Graphical Output
	%  sop=1:  Compute array
	%  sop=2:  Compute running stats (no array)
	%  sop=3:  Compute array and running stats
sop = 2;  % Set sop = 2 normally.
strtplt = 100; % skip first plotted points in statistical plots up to strtplt
%******************************
BnPLL = Wn*(1/(2*z) + 2*z*alpha^2)/4; % Noise BW in Hz

if FoverL % is nonzero
    %computation of effective noise bandwidth in Hz: 
	% Get numerator and denominator of f(s)f(-s), where f is product of loop and filter responses.
	% Filter response is wo/(s+wo).  Loop response is H(s) = N(s)/D(s) from Eq. (6.4a)
	% Find integral of |f(jw)|^2 over jw axis by using residues to get contour integral of f(s)f(-s).
    wo = 4*FoverL*BnPLL; % Unit open-loop gain frequency of filter with designated noise bandwidth, Eq. (14.7)
    nbNum = Wn^4*wo^2*[-(2*alpha*z/Wn)^2 0 1]; % wo^2 N(s)N(-s), numerator of f(s)f(-s)
    nbDn1 = [1 2*z*Wn Wn^2]; % D(s)
    nbDn2 = [1 -2*z*Wn Wn^2]; % D(-s)
    nbDn3 = conv(nbDn1,nbDn2); % D(s)D(-s)
    nbDn4 = [-1 0 wo^2]; % -s^2 + wo = (s + wo)(-s + wo)
    nbDn = conv(nbDn3,nbDn4); % denominator of f(s)f(-s)
    [nbRes,nbPoles,nbRest] = residue(nbNum,nbDn); % [residues, poles, remainder of partial fraction expansion]
    if real(nbPoles(1))<=0 % summing of residues for poles encircled in integration around the left-half plain
        resSum = nbRes(1);
    else
        resSum = 0;
    end
    for i=2:6
        if (real(nbPoles(i))<=0)&&(nbPoles(i)~=nbPoles(i-1))
            resSum = resSum + nbRes(i);
        end
    end
    Bn = abs(resSum/2);
else
    Bn = BnPLL;
end

fprintf('\n\nPHASE STATISTICS (mod 2 pi), Filtered Noise:  NB2(4/13/07)')
fprintf('\n2nd-order loop with SINUSOIDAL Phase Detector Characteristic and 1st-order Noise Filter')
fprintf('\nWn = %g rad/sec; zeta = %g; alpha = %g.  Offset = %g.',Wn,z,alpha,Offset)
if FoverL % is nonzero
    fprintf('\nLoop Noise Bandwidth (linear): %g Hz, Filter Noise Bandwidth: %g Hz, ratio = %g',BnPLL,FoverL*BnPLL,FoverL)
else
    fprintf('\nLoop Noise Bandwidth (linear): %g Hz, Filter Noise Bandwidth is infinite',BnPLL)
end
fprintf('\nNet Noise Bandwidth (linear) Bno = %g Hz.  Number of noise sources: %u\n',Bn,Sources)

%Polynominals for open loop gain:
	N = [0     2*z*Wn*alpha   Wn^2]; % numerator
	D = [1   2*z*Wn*(1-alpha)   0 ]; % denominator

[a,b,c,d] = tf2ss(N,D); % elements of dynamic matrix equation
y = Phinit; % initial phase at output

%Compute unity-gain bandwidth
factor = N(2)^2-D(2)^2;
WL = sqrt((factor+sqrt(factor^2+4*N(3)^2))/2); % open-loop gain = 1

SmpRate = SmpPerWL*WL; % sample rate
FoLMax = SmpRate/BnPLL; % maximum FoverL, where filter is effectively absent
last = 1 + ncmpo; % index of last output
running = last;
if FoverL > FoLMax
    fprintf('\n   Filter ratio (FoverL) exceeds maximum allowed value of %g',FoLMax)
    fprintf('\n   Reset FoverL and repeat.  The maximum is iterative and may be lower at a new setting.\n\n')
    sound(sin(6283/2*[0:1/8192:0.2]),8192);pause(.3);sound(sin(6283/4*[0:1/8192:0.3]),8192);
    running = 0;
end
OutInc = TshowXWL/WL;% period between outputs

% Compute multiplier of noise distribution having unity power.
if NS == 0,
	Dn = No; NS = No*Bn;
	fprintf('Noise Density to Signal Ratio = %g /Hz',No)
else
	Dn = NS/Bn;
 	fprintf('Signal/Noise in Bno = %g.',1/NS)
end
Kn = sqrt(Dn*SmpRate/2); 
if FoverL % is nonzero
    Nfilt = SmpRate/(4*BnPLL*FoverL); % (sampling rate)*(filter time constant = 1/Wo), tau = 1/(4Bn) for one pole
else
    Nfilt = 1;
end
ending = OutInc*ncmpo; % End time
SmpPerOut = SmpRate*OutInc; % samples per output period
t = linspace(0,ending,last); % linear sequence of times for plots
dt = (t(2) - t(1))/SmpPerOut; % sample time increment

if NS < 0.03 % less than 2% error from this approximation or from 14 terms
    sig2 = NS;
else
    sig2 = (pi^2)/3;
    for kk = 1:14
        sig2 = sig2 +4*((-1)^kk/kk^2)*besseli(kk,1/NS)/besseli(0,1/NS);
    end % for
end % if
fprintf('  Variance in Bno is So2 = %g rad^2',NS)
fprintf('\nTheoretical nonlinear variance, 1st order loop, same So2 in Bno: %g rad^2\n',sig2)

% To help judge inaccuracy of ignoring sampling,
%   compute extra phase shift produced at unity gain
SampPhShift = -(90/pi)*WL*OutInc/SmpPerOut; % radians lag from sampling at WL = K
fprintf('Added phase at unity O.L. gain due to sampling = %g deg.\n',SampPhShift)
if truncatePh
	fprintf('Phase will be shown in range 0.5 cycle, modulo 1 cycle.\n')	
else
	fprintf('Phase range will be unrestricted.\n')
end

%    BEGIN SIMULATION
NL = sin(-Phinit); % <<<<<<<<<<<< THE NON-LINEARITY
% state variables to give specified initial conditions
% X2 and X3 respectively in Eq. (6.50) and (6.51) with D(3) = 0:
x0(1) = (Winit-N(2)*(NL+Offset))/(N(3)-D(2)*N(2));
x0(2) = (Phinit-N(2)*x0(1))/N(3);
e = -Phinit; % initial phase error
freq(1) = Winit; % initial frequency at output
Nplus = 0; Nminus = 0; % skipped cycles
Fsum(1) = 0; Fnet(1) = 0; % rate of skipping

if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
	if e>0
		e = rem(e+pi,2*pi) - pi;
	else
		e = rem(e-pi,2*pi) + pi;
	end
end % if truncatePh
ersum = 0; ersum2 = 0;
msv(1) = NS; % start plot at linear value (avoid problem with ratio in dB)
if (sop ~= 2) 
	er(1) = e;
end
[Phi, Gamma] = c2d(a,b,dt); % values for use in solution eqution [see manual]

Noise1 = 0; Noise2 = 0; NoiseUn1 = 0; NoiseUn2 = 0;% filtered and unfiltered noise voltages
vari1 = 0; vari2 = 0; varo1 = 0; varo2 = 0; varo = 0; % noise variances at filter inputs and outputs plus net
i = 1;
while running
    if (running > 10)
        Fnet(last)=0;
        Fsum(last)=0;
        PDnoise(last)=0;
        freq(last)=0;
        mnn(last)=0;
        t(last)=0;
        tshrt(last)=0;
        msv(last)=0;
        varshrt(last)=0;
    end % preassigning arrays
	while i < last, % compute loop responses
        i = i + 1;
        for j = 1:SmpPerOut, % loop through unrecorded samples
            k = (i-2)*SmpPerOut + j + 1;
            NL = sin(-y); % <<<<<<<<<<<< THE NON-LINEARITY 
            NoiseUn1 = randn; NoiseUn2 = randn; %noise at filter input
            vari1 = (NoiseUn1^2 + vari1*(k-1))/k;
            vari2 = (NoiseUn2^2 + vari2*(k-1))/k;
            Noise1 = Noise1*(1-1/Nfilt)+NoiseUn1/Nfilt;
            Noise2 = Noise2*(1-1/Nfilt)+NoiseUn2/Nfilt;
            varo1 = (Noise1^2 + varo1*(k-1))/k;
            varo2 = (Noise2^2 + varo2*(k-1))/k;
            if Sources == 2,
                PDn = Kn*(Noise1*sin(y)+Noise2*cos(y));
            else
                PDn = Kn*Noise1;
            end	
            varo = (PDn^2 + varo*(k-1))/k;
            u1(1) = NL + Offset + PDn; % initial input to sampler
            u1(2) = u1(1); %  constant Hold output -> same at beginning and end of dt
            x = ltitr(Phi,Gamma,u1',x0); % solutions at period start and end [see manual] 
            yy = c*x'; % corresponding outputs (d is null for polynominals above)
            y = yy(2); % output at end of period is needed
            x0 = x(2,:)'; % final condition gives initial condition for next step
            eu = -y/(2*pi);
            if eu - Nplus + Nminus > 1
                Nplus = fix(eu + Nminus);
            elseif eu - Nplus + Nminus < -1
                Nminus = -fix(eu - Nplus);
            end
        end % for j
        PDnoise(i) = PDn;
		e = -y; % untruncated error
		if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
			if e>0
				e = rem(e+pi,2*pi) - pi;
			else
				e = rem(e-pi,2*pi) + pi;
			end
		end % if truncatePh
		if (sop ~= 2)
			er(i) = e;
		end
		if (sop == 2)||(sop == 3)
			ersum = ersum + e;
			ersum2 = ersum2 + e^2;
			msv(i) = ersum2/(i);
			mnn(i) = ersum/(i);
			Fsum(i) = (Nplus + Nminus)/(dt*SmpPerOut*(i-1));
			Fnet(i) = (Nplus - Nminus)/(dt*SmpPerOut*(i-1));
		end
		% corresponding frequency out from Eq. (6.43) with d3 = 0
		freq(i) = N(2)*u1(2) + x0(1)*(N(3)-N(2)*D(2)); 
	end % for i
	
	% END SIMULATION.  The rest is plotting.
	
	clf;
    ncmpo = last - 1;
	ending = ncmpo*OutInc;
	t = linspace(0,ending,last); % reestablish time vector
    
    %****** Start Statistical Plots
    tshrt = t(strtplt:last); varshrt = msv(strtplt:last);
    subplot(221);
    plot([t(strtplt) t(last)],[NS NS],'g',[t(strtplt) t(last)],[sig2 sig2],'m',tshrt,varshrt,'r')
    title('mean square phase (rad^2)')
    grid
    legend('linear','simple (FP)','nonlinear',0)
    subplot(223);
    plot(tshrt,10*log10(varshrt/NS),'b')
    title('rms phase ratio (dB)')
    grid
    subplot(222);
    plot(tshrt,mnn(strtplt:last),'k')
    title('mean phase error (rad)')
    grid
    subplot(224);
    plot(tshrt,Fnet(strtplt:last),'g',tshrt,Fsum(strtplt:last),'r')
    title('skips/sec, total and net')
    grid
    xlabel('seconds')
    % end plotting
    
	if (sop == 1)||(sop == 3) % compute from array
		arvar = std(er)^2;
		logarvar = 10*log10(arvar/NS);
		armnn = mean(er);
		fprintf('\nEstimate from error array:')
        fprintf('\n  output variance %g rad^2\n  ratio to So2 %g dB',arvar,logarvar);
		fprintf('\n  mean %g rad\n  %g points including initial phase\n',armnn,last);
	end % for error array stats
    
	if (sop == 2)||(sop == 3) % compute from running stats
        if Sources == 1
            fprintf('\n  Source 1 variance: %g rad^2',Kn^2*vari1)
            fprintf('\n  Filtered variance: %g rad^2',varo)
        else
            fprintf('\n  Source 1 variance: %g,   Source 2 variance: %g rad^2',Kn^2*vari1, Kn^2*vari2)
            fprintf('\n  Filtered var. 1: %g, Filtered var. 2: %g, Total: %g rad^2',Kn^2*varo1, Kn^2*varo2, varo)
        end
        variance = (msv(last)-(mnn(last))^2)*last/(last-1);
        fprintf('\n  Mean square phase error is variance if mean is known to be zero:')
        fprintf('\n    mean square phase %g rad^2, phase variance %g rad^2',msv(last),variance);
        fprintf('\n  %g skipped cycles with %g net skips',Nplus+Nminus,Nplus-Nminus);
        fprintf('\n  mean phase error %g rad\n  %g points (%g seconds) including initial phase\n',mnn(last),last,last*OutInc);
	end % for running stats
	
	sound(sin(6283*[0:1/8192:0.2]),8192); %0.2 seconds of 1 kHz tone;
	running = input('       Enter additional points (0 to quit, <0 for key in). ');
	if (running < 0)
		running = -running;
		fprintf('\n    Keyboard entry.  Type "return" to resume program.\n')
		keyboard
	end
	last = last + running;
end % while running
%Notes:
%Mean square phase plot: "linear" and "simple (FP)" (Fokker-Plank) are for 0 offset.
%Plotted ratio is (mean square phase out)/(variance in linear loop).