% Time Plots or Phase Variance, mod 2pi, 2nd-Order Loop, Butterworth prefilter
%		plus statistics on skipped cycles.  A cycle is skipped when
%		absolute phase error exceeds 1 cycle.  At that point, the
%		nearest 1-cycle multiple becomes zero phase reference for skips.
%  SINE NONLINEARITY
%  Toolboxes Required: control, signal
% [33 seconds/10,000 points (at 100 samples per output, sop = 2) with Athlon 3000+]
clear;
% MODIFY PARAMETERS BELOW & GO
%*****************************
Limiting = 0; % 0: OFF,  1: ON.  Signal + Noise hard limited.
FN = 2; %Normalize noise in [1 = Filter video BW] or [2 = Loop noise BW] (N/S in Filter Video Bandwidth = PhaseNoise/S in RF BW)
% Set either of the following parameters and set the other to zero.
    NS = 1; % N/S in normalizing BW
    No = 0; % Noise Power Density, per Hz, to Signal Ratio
Sources = 2; % 1 or 2 independent noise sources (not applicable with limiting)
FoverL = 2;  % = r = (Filter Video Bn)/(Loop Bn)
OoverL = -2; % = o = (RF Filter Offset)/(Loop Bn)
Wn = 1;      % rad/sec natural frequency
z = .707;    % damping factor
alpha = .9;   % Eq. (6.5)
SmpPerWL = 100; % ratio of sample rate to unity-gain radian bandwidth 
TshowXWL = 1; % (period between outputs) X (unity-gain radian bandwidth)
ncmpo = 1000;  % computed output points.
Offset = 0;     % offset at PD output (same sign as steady-state output)
Phinit = 0;  % radian initial phase
Winit = 0; % radian/sec initial frequency
truncatePh = 1; % 1:truncate phase (must for statistics); 0:do not.
FLn = Wn/4*(1/2/z + 2*z*alpha^2); Ff = FoverL*FLn; %Loop and filter noise bandwidths (Hz)
%Polynominals for open loop gain:
	N = [0     2*z*Wn*alpha   Wn^2]; % numerator
	D = [1   2*z*Wn*(1-alpha)   0 ]; % denominator
[a,b,c,d] = tf2ss(N,D); % elements of dynamic matrix equation
y = Phinit; % initial phase at output
%Compute unity-gain bandwidth
factor = N(2)^2-D(2)^2;
WL = sqrt((factor+sqrt(factor^2+4*N(3)^2))/2); % open-loop gain = 1
SmpRate = SmpPerWL*WL; % sample rate
dy = OoverL*FLn*2*pi/SmpRate; %radian phase shift per sample due to band offset
%Filter
 forder = 4; % order of Butterworth filter
 fco = 1.95*Ff/SmpRate; % fco*(SmpRate/2) = (BnFilter).  BnFilter correction factor 0.975.
 SRd = ceil(.001/fco); %Sampling rate divisor to prevent problems due to excessive rate/BW ratio
 fco = fco*SRd;
 [bf af] = butter(forder,fco); % Butterworth filter coefficients
[yyy,zf1] = filter(bf, af, randn(1,forder)); % initial filter states
[yyy,zf2] = filter(bf, af, randn(1,forder));
Punits = 0;     % 0:radians  1:cycles  2:degrees
Funits = 0;     % 0:radians/sec   1:Hz
% Choose Graphical Output
%  gop=0:  No plot
%  gop=1:  Phase Plane = Freq. vs. Phase
%  gop=2:  Phase Plane + Phase Error vs. Time
%  gop=3:  Frequency + Sampler Output vs. Time
%  gop=4:  All of above = 4 plots
%  gop=5:  Statistical plots
gop = 5;
strtplt = 1; % skip first plotted points in statistical plots up to strtplt
noplot = 0; % 1 to prevent plotting
%  sop=0:  No stats
%  sop=1:  Compute array
%  sop=2:  Compute running stats (no array)
%  sop=3:  Compute array and running stats
sop = 2;  % Set sop = 2 normally.
if noplot
    sop = 0;
end;
%******************************
if (sop == 2) && (gop > 0) && (gop < 5)
    gop = 5;
end
fprintf('\n\nPHASE STATISTICS (mod 2 pi), Noise through Rectangular filter   NB2bw(4/13/07)')
fprintf('\n2nd-order loop with SINUSOIDAL Phase Detector Characteristic & Butterworth Noise Filter')
fprintf('\nWn = %g rad/sec; zeta = %g; alpha = %g.  Offset = %g.',Wn,z,alpha,Offset);
fprintf('\nPreFilter (%g samples/sec) Video Noise Bandwidth, Bvf = %g Hz.',SmpRate/SRd,Ff);
fprintf('\nLoop (%g samples/sec.) Linear: Noise BW, B0nL = %g Hz.',SmpRate,FLn);
fprintf('\nr = Bvf/B0nL = %g, o = RF Offset/B0nL = %g\n',FoverL, OoverL);
if Limiting
    fprintf('  Hard limiting.\n')
    Sources = 2;
else
    fprintf('  Number of noise sources: %u.  No limiting.\n',Sources)
end
last = 1 + ncmpo; % index of last output
running = last;
OutInc = TshowXWL/WL;% period between outputs

% Compute multiplier of noise distribution having unity power.
if FN == 1
    Fnorm = Ff;
else
    Fnorm = FLn;
end
if NS == 0,
    Dn = No; NS = Fnorm*Dn; 
    fprintf('Noise Density to Signal Ratio = %g /Hz (L = S_phi)',No)
else
    if FN == 1
        Dn = NS/Ff;
        fprintf('Signal/Noise in Bvf = %g.',1/NS);
    else
        Dn = NS/FLn;
        fprintf('Signal/Noise in B0nL = %g.',1/NS);
    end
end
Kn = sqrt(Dn*SmpRate/SRd/2); % noise multiplier to give variance equivalent to density Dn over half of the sampling rate 
ending = OutInc*ncmpo; % End time
SmpPerOut = SmpRate*OutInc; % samples per output period
t = linspace(0,ending,last); % linear sequence of times for plots
dt = (t(2) - t(1))/SmpPerOut; % sample time increment
if gop && (gop < 5)
    clf; % clear previous graphs
end

% Theoretical variance for 1st-order loop, sig2 from Eq. (17.33)
if NS < 0.03 % less than 2% error from this approximation or from 14 terms
    sig2 = NS;
else
    sig2 = (pi^2)/3;
    for kk = 1:14
        sig2 = sig2 +4*((-1)^kk/kk^2)*besseli(kk,1/NS)/besseli(0,1/NS);
    end
end % if

% To help judge inaccuracy of ignoring sampling,
%   compute extra phase shift produced at unity gain
SampPhShift = -(90/pi)*WL*OutInc/SmpPerOut; % radians lag from sampling at WL
fprintf('\nAdded phase at unity O.L. gain due to sampling = %g deg.\n',SampPhShift)
if truncatePh
    fprintf('Phase will be shown in range 0.5 cycle, modulo 1 cycle.\n')	
else
    fprintf('Phase range will be unrestricted.\n')
end

%    BEGIN SIMULATION
NL = sin(-Phinit); % <<<<<<<<<<<< THE NON-LINEARITY
% state variables to give specified initial conditions
% X2 and X3 respectively in Eq. (6.50) and (6.51) with D(3) = 0:
x0(1) = (Winit-N(2)*(NL+Offset))/(N(3)-D(2)*N(2));
x0(2) = (Phinit-N(2)*x0(1))/N(3);
e = -Phinit; % initial phase error
freq(1) = Winit; % initial frequency at output
Nplus = 0; Nminus = 0; % skipped cycles
Fsum(1) = 0; Fnet(1) = 0; % rate of skipping

if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
    if e>0
        e = rem(e+pi,2*pi) - pi;
    else
        e = rem(e-pi,2*pi) + pi;
    end
end % if truncatePh
ersum = 0; ersum2 = 0; 
msv(1) = NS; % start plot at linear value (avoid problem with ratio in dB)
if (sop~= 2) 
    er(1) = e;
end
[Phi, Gamma] = c2d(a,b,dt); % values for use in solution eqution [see MATLAB manual]

vari1 = 0; vari2 = 0; varo1 = 0; varo2 = 0; varo = 0; k2 = 0; % noise variances at filter inputs and outputs plus net
i = 1; 
while running
    if (running > 10) && ~noplot
        Fnet(last)=0;
        Fsum(last)=0;
        PDnoise(last)=0;
        er(last)=0;
        freq(last)=0;
        mnn(last)=0;
        t(last)=0;
        tshrt(last)=0;
        msv(last)=0;
        varshrt(last)=0;
    end % preassigning arrays
    while i < last, % compute loop responses
        i = i + 1;
        for j = 1:SmpPerOut, % loop through unrecorded samples
            k1 = (i-2)*SmpPerOut + j + 1;
            if rem(k1-1.9999,SRd)<0.0002 % sample noise at loop rate / SRd
                k2 = k2 + 1;
                NoiseUn1 = randn; NoiseUn2 = randn; %noise at filter input
                vari1 = (NoiseUn1^2 + vari1*(k2-1))/k2;
                vari2 = (NoiseUn2^2 + vari2*(k2-1))/k2;
                [Noise1, zf1] = filter(bf,af,NoiseUn1,zf1); % noise and filter state from unfiltered noise and previous state
                [Noise2, zf2] = filter(bf,af,NoiseUn2,zf2); 
                varo1 = (Noise1^2 + varo1*(k2-1))/k2;
                varo2 = (Noise2^2 + varo2*(k2-1))/k2;
                if Limiting
                    DelP = atan2(Noise1,Noise2+1/Kn);
                    PDn = 0;
                    varo = (DelP^2 + varo*(k2-1))/k2;
                else
                    if Sources == 2,              
                        PDn = Kn*(Noise1*sin(y - k1*dy)+Noise2*cos(y - k1*dy)); %dy sign change 11/25/06 
                    else
                        PDn = Kn*Noise1;
                    end
                    varo = (PDn^2 + varo*(k2-1))/k2;
                    DelP = 0;
                end
            end % if noise sample
            NL = sin(DelP-y); % <<<<<<<<<<<< THE NON-LINEARITY 
            u1(1) = NL + Offset + PDn; % initial input to sampler
            u1(2) = u1(1); %  constant Hold output -> same at beginning and end of dt
            x = ltitr(Phi,Gamma,u1',x0); % solutions at period start and end [see manual] 
            yy = c*x'; % corresponding outputs (d is null for polynominals above)
            y = yy(2); % output at end of period is needed
            x0 = x(2,:)'; % final condition gives initial condition for next step
            % Count cycle skips
            eu = -y/(2*pi);
            if eu - Nplus + Nminus > 1
                Nplus = fix(eu + Nminus);
            elseif eu - Nplus + Nminus < -1
                Nminus = -fix(eu - Nplus);
            end
        end % for j			
        if ~noplot
            PDnoise(i) = DelP + PDn;
        end
        e = -y; % untruncated error
        if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
            if e>0
                e = rem(e+pi,2*pi) - pi;
            else
                e = rem(e-pi,2*pi) + pi;
            end
        end % if truncatePh
        if (sop ~= 2)
            er(i) = e;
        end
        if (sop == 2)||(sop == 3)
            ersum = ersum + e;
            ersum2 = ersum2 + e^2;
            if ~noplot
                msv(i) = ersum2/(i);
                mnn(i) = ersum/(i);
                Fsum(i) = (Nplus + Nminus)/(dt*SmpPerOut*(i-1));
                Fnet(i) = (Nplus - Nminus)/(dt*SmpPerOut*(i-1));
            end
        end
        % corresponding frequency out from Eq. (6.43) with d3 = 0
        if ~noplot
            freq(i) = N(2)*u1(2) + x0(1)*(N(3)-N(2)*D(2)); 
        end
    end % for i
    
    % END SIMULATION.  The rest is plotting.
    
    clf;
    ncmpo = last - 1;
    ending = ncmpo*OutInc;
    if ~noplot
        t = linspace(0,ending,last); % reestablish time vector
    end
    
    %****** Start Response Plots for Diagnostics Use	
    if Punits == 1,
        Pscale = 1/(2*pi);
    elseif Punits == 2,
        Pscale = 180/pi;
    else
        Pscale = 1;
    end % if Punits
    
    if Funits == 1,
        Fscale = 1/(2*pi);
    else
        Fscale = 1;
    end % if Funits
    
    if (gop > 0)&&(gop < 5)
        if (gop == 1)||(gop == 2)||(gop == 4), % selecting phase-plane plot
            if gop==1,
                subplot(111),
            elseif gop==2,
                subplot(211),
            elseif gop==4,
                subplot(221),
            end % if gop==1
            plot(Pscale*(er),Fscale*freq,'r')
            title('Phase Plane, Nonlinear Simulation')
            grid
            if Punits == 1, % Set display of phase units
                xlabel('phase error (cycles)');
            elseif Punits == 2,
                xlabel('phase error (degrees)');
            else
                xlabel('phase error (radians)')
            end % if Punits
            if Funits == 1, % Set display of frequency units
                ylabel('freq. out (Hz)');
            else
                ylabel('freq. out (rad/sec)');
            end % if Funits
        end % if gop ~= 0 or 3
        
        if (gop == 2)||(gop == 4), % selecting phase error vs. time
            if gop == 2,
                subplot (212),
            else
                subplot (223),
            end % if gop==2
            plot(t,Pscale*er,'g')
            title('Phase Error')
            grid
            xlabel('seconds')
            if Punits == 1, % Set display of phase units
                ylabel('cycles');
            elseif Punits == 2,
                ylabel('degrees');
            else
                ylabel('radians');
            end % if Punits
        end % if ()|()
        
        if (gop == 3)||(gop == 4), 
            if gop == 3, % selecting frequency vs. time
                subplot(211);
            else
                subplot(222),
            end % if gop==3 
            plot(t,Fscale*freq,'b')
            title('Frequency Out')
            grid
            xlabel('seconds');
            if Funits == 1, % Set display of frequency units
                ylabel('Hz');
            else
                ylabel('radians / second');
            end % if Funits
            
            if gop == 3, % selecting Sampler Output vs. time
                subplot(212);
            else
                subplot(224),
            end % if gop==3 
            if Limiting
                plot(t,sin(er+PDnoise),'+')
            else
                plot(t,sin(er)+PDnoise,'+')
            end
            title('Sampler Output')
            grid
            xlabel('seconds')
        end % if ()|()
    elseif (gop == 5)&&((sop == 2)||(sop == 3))
        %****** Start Statistical Plots	
        if ~noplot
            tshrt = t(strtplt:last); varshrt = msv(strtplt:last);
            subplot(221);
            plot(tshrt,varshrt-mnn(strtplt:last).^2,'r')
            title('phase sample variance (rad^2)')
            grid
            %       legend('linear','simple (FP)','nonlinear',0)
            subplot(223);
            plot(tshrt,varshrt,'r')
            title('mean square phase (rad^2)')
            grid
            subplot(222);
            plot(tshrt,mnn(strtplt:last),'k')
            title('mean phase error (rad)')
            grid
            subplot(224);
            plot(tshrt,Fnet(strtplt:last),'g',tshrt,Fsum(strtplt:last),'r')
            title('skips/sec, total and net')
            grid
            xlabel('seconds')
        end
    end % plotting
    
    if (sop == 1)||(sop == 3) % compute from array
        arvar = std(er)^2;
        logarvar = 10*log10(arvar/NS);
        armnn = mean(er);
        fprintf('\nEstimate from error array:')
        fprintf('\n  output variance %g rad^2\n  ratio to So2 %g dB',arvar,logarvar);
        fprintf('\n  mean phase error %g rad\n  %g points including initial phase\n',armnn,last);
    end % for error array stats
    
    if (sop == 2)||(sop == 3) % compute from running stats
        if Sources == 1
            fprintf('\n  Source 1 variance: %g rad^2',Kn^2*vari1)
            fprintf('\n  Filtered variance: %g rad^2',varo)
        else
            fprintf('\n  Source 1 variance: %g,   Source 2 variance: %g rad^2',Kn^2*vari1, Kn^2*vari2)
            fprintf('\n  Filtered var. 1: %g, Filtered var. 2: %g, Total: %g rad^2',Kn^2*varo1, Kn^2*varo2, varo)
        end
        if ~Limiting
            fprintf('\n    (Note: these rad units are equivalents using linear Kp.)')
        end
        Nskip = Nplus+Nminus; Dur = last*OutInc;
        meanph = ersum/i; variance = (ersum2 - ersum^2/i)/(i-1);
        fprintf('\n  (Filtered var. 1)/Specified = %g. Output variance %g rad^2',Kn^2*varo1/NS,variance);
        fprintf('\n  mean phase error %g rad.  %g skipped cycles with %g net skips.',meanph,Nskip,Nplus-Nminus);
        fprintf('\n  F/(RF video BW) = %g.  F/(Loop noise BW) = %g.',Nskip/Dur/Ff,Nskip/Dur/FLn);
        fprintf('\n  %g points (%g seconds) including initial phase\n', last, Dur);        
    end % for running stats
    
    sound(sin(6283*[0:1/8192:0.2]),8192); %0.2 seconds of 1 kHz tone;
    running = input('    Enter additional points (0 to quit, <0 for key in). ');
    if (running < 0)
        running = -running;
        fprintf('\n    Keyboard entry.  Type "return" to resume program.\n')
        keyboard
    end
    last = last + running;
end % while running
