% Time Responses or Phase Variance, mod 2pi, Second-Order Loop
%		plus statistics on skipped cycles.  A cycle is skipped when
%			absolute phase error exceeds 1 cycle.  At that point, the
%			nearest 1-cycle multiple becomes zero phase reference for skips.
%   SINE NONLINEARITY
% Toolboxes Required: none
%[14,300 points/second, 70 seconds/million with Athlon 3000+, sop=2]
clear;

% MODIFY PARAMETERS BELOW & GO
%*****************************
% Set either of the following parameters and set the other to zero.
  NS = 0.5; % N/S in Loop Bandwidth
  No = 0; % Noise Power Density, per Hz, to Signal Ratio
Sources = 2; % 1 or 2 independent noise sources
Phinit = 0;  % radian initial phase
K = 1;      % rad/sec unity-gain frequency
SmpRate = 100*K; % sample rate
OutInc = 1/K;% period between outputs
ncmpo = 10000;  % computed output points.
Offset = 0;     % offset at PD output (same sign as steady-state output)
truncatePh = 1; % 1:truncate phase (must for statistics); 0:do not.
Punits = 0;     % 0:radians  1:cycles  2:degrees
Funits = 0;     % 0:radians/sec   1:Hz
		% Choose Graphical Output
		%  gop=0:  No plot
		%  gop=1:  Phase Plane = Freq. vs. Phase
		%  gop=2:  Phase Plane + Phase Error vs. Time
		%  gop=3:  Frequency + Sampler Output vs. Time
		%  gop=4:  All of above = 4 plots
		%  gop=5:  Statistical plots
		gop = 4;
	%  sop=0:  No stats
	%  sop=1:  Compute array of errors
	%  sop=2:  Compute running stats (no array)
	%  sop=3:  Compute array and running stats (compare results)
sop = 2;  % Set sop = 2 normally.
strtplt = 1000; % skip first plotted points in statistical plots up to strtplt
%******************************
	if (sop == 2) && (gop > 0) && (gop < 5)
		gop = 5;
	end

fprintf('\n\nPHASE STATISTICS (mod 2 pi)    NStat1(4/12/07)')
fprintf('\n1st-Order Loop with SINUSOIDAL Phase Detector Characteristic')
fprintf('\nK = %g rad/sec = closed-loop BW = open-loop unity-gain frequency',K)
Bn =K/4; % Noise BW in Hz
fprintf('\nOffset is %g.  Number of noise sources: %u',Offset,Sources)
fprintf('\nNoise Bandwidth (linear), Bno = %g Hz.\n',Bn)
% Compute multiplier Kn of noise distribution having unity power.
if NS == 0,
	Dn = No; NS = Bn*Dn; 
	fprintf('Noise Density to Signal Ratio = %g /Hz',No)
else
	Dn = NS/Bn;
 	fprintf('Signal/Noise in Bno = %g',1/NS)
end
Kn = sqrt(Dn*SmpRate/2); 
last = 1 + ncmpo; % index of last output
ending = OutInc*ncmpo; % End time
SmpPerOut = SmpRate*OutInc; % samples per output period
t = linspace(0,ending,last); % linear sequence of times for plots
dt = (t(2) - t(1))/SmpPerOut; % sample time increment
if gop && (gop < 5)
	clf;
end

% Theoretical variance for 1st-order loop, sig2 from Eq. (17.33)
if NS < 0.03 % less than 2% error from this approximation
    sig2 = NS;
else % less than  2% error from 14 terms
    sig2 = (pi^2)/3;
    for kk = 1:14
        sig2 = sig2 +4*((-1)^kk/kk^2)*besseli(kk,1/NS)/besseli(0,1/NS);
    end
end % if
fprintf('  Variance in Bno is So2 = %g rad^2',NS)
fprintf('\nTheoretical nonlinear variance, 1st order loop, same So2 in Bno: %g rad^2\n',sig2)

% To help judge inaccuracy of ignoring sampling,
%   compute extra phase shift produced at unity gain
SampPhShift = -(90/pi)*K*OutInc/SmpPerOut; % radians lag from sampling at WL = K
fprintf('Added phase at unity O.L. gain due to sampling = %g deg.\n',SampPhShift)
if truncatePh
	fprintf('Phase will be shown in range 0.5 cycle, modulo 1 cycle.\n')	
else
	fprintf('Phase range will be unrestricted.\n')
end

if (NS ~= 0)
    Nstats = 1;
else
    Nstats = 0;
end % conditions OK for noise statistics
if ~Nstats
    fprintf('\nVariances not computed because of zero noise.\n')
else
    fprintf('\n')
end

%    BEGIN SIMULATION
y = Phinit; % initial phase at output
NL = sin(-Phinit); % <<<<<<<<<<<< THE NON-LINEARITY
e = -Phinit; % initial phase error
freq(1) = K*NL; % initial frequency at output
Nplus = 0; Nminus = 0; % skipped cycles
Fsum(1) = 0; Fnet(1) = 0; % rate of skipping

if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
	if e>0
		e = rem(e+pi,2*pi) - pi;
	else
		e = rem(e-pi,2*pi) + pi;
	end
end % if truncatePh
ersum = 0; ersum2 = 0; 
msv(1) = NS; % start plot at linear value (avoid problem with ratio in dB)
if (sop~= 2) 
	er(1) = e;
end

running = last; i = 1;
while running
    if (running > 10)
        Fnet(last)=0;
        Fsum(last)=0;
        PDnoise(last)=0;
        if (sop~=2)
            er(last) = 0;
        end
        freq(last)=0;
        mnn(last)=0;
        t(last)=0;
        tshrt(last)=0;
        msv(last)=0;
        varshrt(last)=0;
    end % preassigning arrays
	while i < last, % compute loop responses
		i = i + 1;
		for j = 1:SmpPerOut, % loop through unrecorded samples
			NL = sin(-y); % <<<<<<<<<<<< THE NON-LINEARITY 
			if Sources == 2,
                Noise1 = randn; Noise2 = randn;
                PDn = Kn*(Noise1*sin(y)+Noise2*cos(y));
            else
                PDn = Kn*randn;
            end
			u1 = NL + Offset + PDn; % initial input to sampler = constant Hold output -> same at beginning and end of dt
			y = y + u1*K*dt;
            %Record cycle skips
            eu = -y/(2*pi); % phase in cycles
			if eu - Nplus + Nminus > 1
				Nplus = fix(eu + Nminus);
			elseif eu - Nplus + Nminus < -1
				Nminus = -fix(eu - Nplus);
			end
		end % for j			
        PDnoise(i) = PDn;
		e = -y; % untruncated error
		if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
			if e>0
				e = rem(e+pi,2*pi) - pi;
			else
				e = rem(e-pi,2*pi) + pi;
			end
		end % if truncatePh
		if (sop ~= 2)
			er(i) = e;
		end
		if (sop == 2)||(sop == 3)
			ersum = ersum + e;
			ersum2 = ersum2 + e^2;
			msv(i) = ersum2/(i); %msv(i) is variance for zero mean, otherwise it is mean square
			mnn(i) = ersum/(i);
			Fsum(i) = (Nplus + Nminus)/(dt*SmpPerOut*(i-1));
			Fnet(i) = (Nplus - Nminus)/(dt*SmpPerOut*(i-1));
		end
		% corresponding frequency out from Eq. (6.43) with d3 = 0
		freq(i) = K*u1; 
	end % for i
	
	% END SIMULATION.  The rest is plotting.
	
	clf;
    ncmpo = last - 1;
	ending = ncmpo*OutInc;
	t = linspace(0,ending,last); % reestablish time vector

%****** Start Response Plots for Diagnostics Use	
	if Punits == 1,
		Pscale = 1/(2*pi);
	elseif Punits == 2,
		Pscale = 180/pi;
    else
		Pscale = 1;
	end % if Punits
	
	if Funits == 1,
		Fscale = 1/(2*pi);
	else
		Fscale = 1;
	end % if Funits
	
	if (gop > 0)&&(gop < 5)
		if (gop == 1)||(gop == 2)||(gop == 4), % selecting phase-plane plot
			if gop==1,
				subplot(111),
			elseif gop==2,
				subplot(211),
			elseif gop==4,
				subplot(221),
			end % if gop==1
			plot(Pscale*(er),Fscale*freq,'r')
			title('Phase Plane, Nonlinear Simulation')
			grid
			if Punits == 1, % Set display of phase units
				xlabel('phase error (cycles)');
			elseif Punits == 2,
				xlabel('phase error (degrees)');
            else
				xlabel('phase error (radians)')
			end % if Punits
			if Funits == 1, % Set display of frequency units
				ylabel('freq. out (Hz)');
            else
			 ylabel('freq. out (rad/sec)');
			end % if Funits
		end % if gop ~= 0 or 3
		
		if (gop == 2)||(gop == 4), % selecting phase error vs. time
			if gop == 2,
				subplot (212),
            else
				subplot (223),
			end % if gop==2
			plot(t,Pscale*er,'g')
			title('Phase Error')
			grid
			xlabel('seconds')
			if Punits == 1, % Set display of phase units
				ylabel('cycles');
			elseif Punits == 2,
				ylabel('degrees');
            else
				ylabel('radians');
			end % if Punits
		end % if ()|()
		
		if (gop == 3)||(gop == 4), 
			if gop == 3, % selecting frequency vs. time
				subplot(211);
            else
				subplot(222),
			end % if gop==3 
			plot(t,Fscale*freq,'b')
			title('Frequency Out')
			grid
			xlabel('seconds');
			if Funits == 1, % Set display of frequency units
				ylabel('Hz');
            else
			 ylabel('radians / second');
			end % if Funits
			if gop == 3, % selecting Sampler Output vs. time
				subplot(212);
            else
				subplot(224),
			end % if gop==3 
			plot(t,sin(er)+PDnoise,'+')
			title('Sampler Output')
			grid
			xlabel('seconds')
		end % if ()|()
	elseif (gop == 5)&&((sop == 2)||(sop == 3))
%****** Start Statistical Plots	
		tshrt = t(strtplt:last); varshrt = msv(strtplt:last);
		subplot(221);
		plot([t(strtplt) t(last)],[NS NS],'g',[t(strtplt) t(last)],[sig2 sig2],'m',tshrt,varshrt,'r')
		title('mean square phase (rad^2)')
		grid
		legend('linear','simple (FP)','nonlinear',0)
		subplot(223);
        if Nstats
            plot(tshrt,10*log10(varshrt/NS),'b')
            title('rms phase ratio (dB)')
            grid
        end %if nstat
		subplot(222);
		plot(tshrt,mnn(strtplt:last),'k')
		title('mean phase error (rad)')
		grid
		subplot(224);
		plot(tshrt,Fnet(strtplt:last),'g',tshrt,Fsum(strtplt:last),'r')
		title('skips/sec, total and net')
		grid
		xlabel('seconds')
	end % plotting
	
	if (sop == 1)||(sop == 3) % compute from array
		arvar = std(er)^2;
		logarvar = 10*log10(arvar/NS);
		armnn = mean(er);
		fprintf('\nEstimate from error array:')
        fprintf('\n  output variance %g rad^2\n  ratio to So2 %g dB',arvar,logarvar);
		fprintf('\n  mean phase error %g rad\n  %g points including initial phase\n',armnn,last);
	end % for error array stats
    
	if (sop == 2)||(sop == 3) % compute from running stats
        fprintf('Mean square phase error is variance if mean is known to be zero:')
        if Nstats
            variance = (msv(last)-(mnn(last))^2)*last/(last-1);
            fprintf('\n  mean square phase %g rad^2, phase variance %g rad^2',msv(last),variance);
        end
		fprintf('\n  %g skipped cycles with %g net skips',Nplus+Nminus,Nplus-Nminus);
		fprintf('\n  mean phase error %g rad',mnn(last));
		fprintf('\n  %g points (%g seconds) including initial phase\n',last,last*OutInc);
	end % for running stats
	
	sound(sin(6283*[0:1/8192:0.2]),8192); %0.2 seconds of 1 kHz tone;
	running = input('    Enter additional points (0 to quit, <0 for key in). ');
	if (running < 0)
		running = -running;
		fprintf('\n    Keyboard entry.  Type "return" to resume program.\n')
		keyboard
	end
	last = last + running;
end % while running
%Notes:
%Mean square phase plot: "linear" and "simple (FP)" (Fokker-Plank) are for 0 offset.
%Plotted ratio is (mean square phase out)/(variance in linear loop).