%Develop error step response by standard matrix procedure
%  alpha = 1, zeta = 0.5, Wn = 1
%  Step Input
%  Toolbox required: control

% Generate vector of computation times:
inc = .05; %sampling period
ending = 10;
last = 1 + ending/inc; % index of last data points
t = linspace(0,ending,last); % linear sequence of times
dt = t(2) - t(1); % time increment

% Generate input step vector:
u = ones(1,last); % unit step input; column vector with input at each time

clf; % clear previous graphs

NUM = [1 0 0]; % for error response, as before
DEN = [1 1 1]; % denstep from ErTrn1a
[a,b,c,d] = tf2ss(NUM,DEN); % get elements of dynamic matrix equation [c is bottom row of C and d is bottom row of D in Eq. (6.44)]
[Phi, Gamma] = C2D(a,b,dt); % Continuous-to-Discrete, gets values for use in solution equation
x = ltitr(Phi,Gamma,u'); % state variable solution for each value of t
y = c*x'+d*u; % outputs.  x is transpose of X in Eq. (6.33); y is lower element in Eq. (6.45)
    % x',y and u have a column for each time increment

plot(t,y) 
grid
title('Time Response, Error, Step Input')
xlabel('seconds')
ylabel('radians')

		
		

