% Phase Variance, mod 2pi, First-Order Loop with input phase modulation
%		plus statistics on skipped cycles.  A cycle is skipped when
%		absolute phase error exceeds 1 cycle.  At that point, the
%		nearest 1-cycle multiple becomes zero phase reference for skips.  
%   SINE NONLINEARITY
% Toolboxes Required: none
%[11,000 points/second, 90 seconds/million with Athlon 3000+ (sop = 2)]
clear;
for NS = [1 2]
for Sources = [2]
clear Fnet Fsum PDnoise freq mnn t tshrt var varshrt 

% MODIFY PARAMETERS BELOW & GO
%*****************************
% Set either of the following parameters and set the other to zero.
%NS = .05; % N/S in Loop Bandwidth
No = 0; % Noise Power Density, per Hz, to Signal Ratio
%Sources = 2; % 1 or 2 independent noise sources

% Set input phase modulation
PkDelPhi = 30; % degrees peak deviation, square wave
MPeriod = 8; % sample periods per modulation period, even number, not submultiple of SmpRate/K

Phinit = 0;  % radian initial phase without modulation
K = 1;      % rad/sec unity-gain frequency
SmpRate = 100*K; % sample rate
OutInc = 1/K;% period between outputs
ncmpo = 10000;  % computed output points.
Offset = 0;     % offset at PD output (same sign as steady-state output)
truncatePh = 1; % 1:truncate phase (must for statistics); 0:do not.
		% Choose Graphical Output
strtplt = 1000; % skip first plotted points in statistical plots up to strtplt
	%  sop=1:  Compute array
	%  sop=2:  Compute running stats (no array)
	%  sop=3:  Compute array and running stats
sop = 2;  % Set sop = 2 normally.
%******************************

fprintf('\n\nFirst-Order Loop with Modulation, iterative: Sig1i (4/13/07)')
fprintf('\nPHASE STATISTICS (mod 2 pi) WITH HIGH-FREQUENCY PM')
fprintf('\n1st-Order Loop with SINUSOIDAL Phase Detector Characteristic')
fprintf('\nK = %g rad/sec',K)
Bn =K/4; % Noise BW in Hz
fprintf('\nNoise Bandwidth (linear) = %g Hz.  Number of noise sources: %u\n',Bn,Sources)
% Compute multiplier of noise distribution having unity power.
if NS == 0,
	Dn = No; NS = Bn*Dn;
	fprintf('Noise Density to Signal Ratio = %g /Hz',No)
else
	Dn = NS/Bn;
 	fprintf('Signal/Noise in Bno = %g',1/NS)
end
fprintf('\nOffset %g = asin(%g rad)',Offset,asin(Offset))
fprintf('\n%g degrees modulation at %g/sec.',PkDelPhi,SmpRate/MPeriod)
Kn = sqrt(Dn*SmpRate/2); % noise multiplier to set density

last = 1 + ncmpo; % index of last output
ending = OutInc*ncmpo; % End time
SmpPerOut = SmpRate*OutInc; % samples per output period
t = linspace(0,ending,last); % linear sequence of times for plots
dt = (t(2) - t(1))/SmpPerOut; % sample time increment
PkDelRad = PkDelPhi*pi/180; % peak phase deviation in radians
if NS < 0.03 % less than 2% error from this approximation or from 14 terms
    sig2 = NS;
else
    sig2 = (pi^2)/3;
    for kk = 1:14
        sig2 = sig2 +4*((-1)^kk/kk^2)*besseli(kk,1/NS)/besseli(0,1/NS);
    end % for
end % if
fprintf('  Variance in Bno is So2 = %g rad^2',NS)
fprintf('\nTheoretical variance, 1st order loop, no mod. or offset, same So2 in Bno: %g rad^2\n',sig2)

% To help judge inaccuracy of ignoring sampling,
%   compute extra phase shift produced at unity gain
SampPhShift = -(90/pi)*K*OutInc/SmpPerOut; % radians lag from sampling at WL = K
fprintf('Added phase at unity O.L. gain due to sampling = %g deg.\n',SampPhShift)
if truncatePh
    fprintf('Phase will be shown in range 0.5 cycle, modulo 1 cycle.')	
else
    fprintf('Phase range will be unrestricted.')
end
if (Phinit == 0 && Offset == 0 && NS ~= 0)
    Nstats = 1;
else
    Nstats = 0;
end % conditions OK for noise statistics with known zero mean

if ~Nstats
    fprintf('\n\nVariances not computed because of zero noise or non-zero mean phase.\n')
else
    fprintf('\n')
end


%    BEGIN SIMULATION
y = Phinit; % initial phase at output
e = - Phinit; % initial phase error
NL = sin(e); % <<<<<<<<<<<< THE NON-LINEARITY

freq(1) = K*NL; % initial frequency at output
Nplus = 0; Nminus = 0; % skipped cycles
Fsum(1) = 0; Fnet(1) = 0; % rate of skipping

if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
	if e>0
		e = rem(e+pi,2*pi) - pi;
	else
		e = rem(e-pi,2*pi) + pi;
	end
end % if truncatePh
ersum = 0; ersum2 = 0; 
var(1) = NS; % start plot at linear value (avoid problem with ratio in dB)
if (sop ~= 2) 
	er(1) = e;
end

running = last; i = 1;
while running
    if (running > 10)
        Fnet(last)=0;
        Fsum(last)=0;
        PDnoise(last)=0;
        freq(last)=0;
        mnn(last)=0;
        t(last)=0;
        tshrt(last)=0;
        var(last)=0;
        varshrt(last)=0;
    end % preassigning arrays
    ModBias = (MPeriod-1)/2;
	while i < last, % compute loop responses
		i = i + 1;
        Tsamples = (i-2)*SmpPerOut; % total samples used to here
        ModStart = Tsamples -.9 + round(MPeriod/4); % start modulation at 1/4 period
		for j = 1:SmpPerOut, % loop through unrecorded samples
			DelRad = PkDelRad * sign(ModBias - rem(j+ModStart,MPeriod));
%            if (i<4)
%                DRSamp(i,j) = DelRad;
%            end
%            if (i>56) & (i<60)
%                DRSamp(i-52,j) = DelRad;
%            end
			NL = sin(DelRad - y); % <<<<<<<<<<<< THE NON-LINEARITY 
			if Sources == 2,
                Noise1 = randn; Noise2 = randn;
                PDn = Kn*(Noise1*sin(y)+Noise2*cos(y));
            else
                PDn = Kn*randn;
            end
			u1 = NL + Offset + PDn; % initial input to sampler = constant Hold output -> same at beginning and end of dt
			y = y + u1*K*dt;eu = -y/(2*pi);
			%Count cycle skips
            if eu - Nplus + Nminus > 1
				Nplus = fix(eu + Nminus);
			elseif eu - Nplus + Nminus < -1
				Nminus = -fix(eu - Nplus);
			end
		end % for j
        PDnoise(i) = PDn;
		e = -y; % untruncated error, carrier in minus output
		if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
			if e>0
				e = rem(e+pi,2*pi) - pi;
			else
				e = rem(e-pi,2*pi) + pi;
			end
		end % if truncatePh
		if (sop ~= 2)
			er(i) = e + DelRad;
		end
		if (sop == 2)||(sop == 3)
			ersum = ersum + e;
			ersum2 = ersum2 + e^2;
			var(i) = ersum2/(i);
			mnn(i) = ersum/(i);
			
			Fsum(i) = (Nplus + Nminus)/(dt*(i-1));
			Fnet(i) = (Nplus - Nminus)/(dt*(i-1));
		end
		% corresponding frequency out from Eq. (6.43) with d3 = 0
		freq(i) = K*u1; 
	end % for i
	
	% END SIMULATION.  The rest is plotting.
    
	clf;
	ncmpo = last - 1;
	ending = ncmpo*OutInc;
	t = linspace(0,ending,last); % reestablish time vector

%****** Start Statistical Plots	
		tshrt = t(strtplt:last); varshrt = var(strtplt:last);
        if Nstats
            subplot(411);
            plot([t(strtplt) t(last)],[NS NS],'g',[t(strtplt) t(last)],[sig2 sig2],'m',tshrt,varshrt,'r')
            title('phase variance (rad^2)')
            grid
            legend('linear','theoretical','nonlinear',0)
            subplot(412);
            plot(tshrt,10*log10(varshrt/NS),'b')
            title('variance ratio (dB)')
            grid
        end % zero initial conditions
		subplot(413);
		plot(tshrt,mnn(strtplt:last),'k')
		title('mean phase error (rad)')
		grid
		subplot(414);
		plot(tshrt,Fnet(strtplt:last),'g',tshrt,Fsum(strtplt:last),'r')
		title('skip frequency (skips/sec), total and net')
		grid
		xlabel('seconds')
	% end plotting
	
    if (sop == 1)||(sop == 3) % compute from array
        fprintf('\nEstimate from error array:')
        if Nstats
            arvar = std(er)^2;
            logarvar = 10*log10(arvar/NS);
            fprintf('\n  output variance %g rad^2\n  ratio to So2 %g dB',arvar,logarvar);
        end % if Nstats
        armnn = mean(er);
        fprintf('\n  mean phase error %g rad\n  %g points including initial phase\n',armnn,last);
    end % for error array stats
    
    if (sop == 2)||(sop == 3) % compute from running stats
        fprintf('\nRunning statistics for known zero mean:')
        if Nstats
            logvar = 10*log10(var(last)/NS);
            fprintf('\n  output variance %g rad^2\n  ratio to So2 %g dB',var(last),logvar);
        end
        fprintf('\n  %g skipped cycles with %g net skips',Nplus+Nminus,Nplus-Nminus);
		fprintf('\n  mean phase error %g rad\n  %g points (%g seconds) including initial phase\n',mnn(last),last,last*OutInc);
    end % for running stats
	running = 0;
	sound(sin(6283*[0:1/8192:0.2]),8192); %0.2 seconds of 1 kHz tone;
    pause(1);
end % while running
end % for Sources
end % for NS