% Phase Variance, mod 2pi, Second-Order Loop with input phase modulation
%		plus statistics on skipped cycles.  A cycle is skipped when
%			absolute phase error exceeds 1 cycle.  At that point, the
%			nearest 2pi multiple becomes zero phase reference for skips. 
%  SINE NONLINEARITY
% Toolboxes Required: control
% [16,900 points/minute, 35.5 seconds/10000 points (at 100 samples per output, sop=2) with Athlon 3000+]
clear;
for Offset = [.2667 .1481] %variable 1
for NS = [1 .5] %variable 2
clear Fnet Fsum PDnoise freq mnn t tshrt msv varshrt 
fid = fopen('runrec.txt','a');

% MODIFY PARAMETERS BELOW & GO
%*****************************
% Set either of the following parameters and set the other to zero.
%NS = .5; % N/S in Loop Bandwidth
No = 0; % Noise Power Density, per Hz, to Signal Ratio
Sources = 2; % 1 or 2 independent noise sources

% Set input phase modulation
PkDelPhi = 0; % degrees peak deviation, square wave
MPeriod = 8; % sample periods per modulation period, even number, not submultiple of SmpRate/K

Phinit = 0;  % radian initial phase
Winit = 0; % radian/sec initial frequency
Wn = 1;      % rad/sec natural frequency
z = 1;    % damping factor
alpha = 0.985;   % Eq. (6.5)
SmpPerWL = 100; % ratio of sample rate to unity-gain radian bandwidth 
TshowXWL = 1; % (period between outputs) X (unity-gain radian bandwidth)
ncmpo = 4000;  % computed output points.
%Offset = 0;     % offset at PD output (same sign as steady-state output)
truncatePh = 1; % 1:truncate phase; 0:do not
Punits = 0;     % 0:radians  1:cycles  2:degrees
Funits = 0;     % 0:radians/sec   1:Hz
		% Choose Graphical Output
		%  gop=0:  No plot
		%  gop=1:  Phase Plane = Freq. vs. Phase
		%  gop=2:  Phase Plane + Phase Error vs. Time
		%  gop=3:  Frequency + Sampler Output vs. Time
		%  gop=4:  All of above = 4 plots
		%  gop=5:  Statistical plots
		gop = 5;
strtplt = 11; % skip first plotted points in statistical plots up to strtplt
	%  sop=0:  No stats
	%  sop=1:  Compute array
	%  sop=2:  Compute running stats (no array)
	%  sop=3:  Compute array and running stats
sop = 2;  % Set sop = 2 normally.
%******************************
	if (sop == 2) && (gop > 0) && (gop < 5)
		gop = 5;
	end

fprintf('\n\nIteractive version    SigOff2i (4/13/07)')
fprintf('\nPHASE STATISTICS (mod 2 pi) With HIGH-FREQUENCY PM and PHASE OFFSET')
fprintf('\n2nd-order loop with SINUSOIDAL Phase Detector Characteristic')
fprintf('\nWn = %g rad/sec; zeta = %g; alpha = %g',Wn,z,alpha)
Bn = Wn*(1/(2*z) + 2*z*alpha^2)/4; % Noise BW in Hz
fprintf('\nNoise Bandwidth = %g Hz.  Number of noise sources: %u',Bn,Sources)
fprintf(fid,'\n\nIteractive version of SigOff2    SigOff2i (4/13/07)');
fprintf(fid,'\nPHASE STATISTICS (mod 2 pi) With HIGH-FREQUENCY PM and PHASE OFFSET');
fprintf(fid,'\n2nd-order loop with SINUSOIDAL Phase Detector Characteristic');
fprintf(fid,'\nWn = %g rad/sec; zeta = %g; alpha = %g',Wn,z,alpha);
fprintf(fid,'\nNoise Bandwidth = %g Hz.  Number of noise sources: %u',Bn,Sources);
if alpha ~= 1
    if alpha == 0
        kOverWn = 1/2/z;
    else
        kOverWn = (1+1/(1/alpha-1))/2/z;
    end
    fprintf('\nOffset is %g, producing %g Wn offset in center frequency\n',Offset,Offset*kOverWn)
    fprintf(fid,'\nOffset is %g, producing %g Wn offset in center frequency\n',Offset,Offset*kOverWn);
else
    fprintf('\nOffset is %g, producing infinite offset in center frequency\n',Offset)
    fprintf(fid,'\nOffset is %g, producing infinite offset in center frequency\n',Offset);
end

%Polynominals for open loop gain:
	N = [0     2*z*Wn*alpha   Wn^2]; % numerator
	D = [1   2*z*Wn*(1-alpha)   0 ]; % denominator

[a,b,c,d] = tf2ss(N,D); % elements of dynamic matrix equation
y = Phinit; % initial phase at output

%Compute unity-gain bandwidth
factor = N(2)^2-D(2)^2;
WL = sqrt((factor+sqrt(factor^2+4*N(3)^2))/2); % open-loop gain = 1

SmpRate = SmpPerWL*WL; % sample rate
OutInc = TshowXWL/WL;% period between outputs

% Compute multiplier of noise distribution having unity power.
if NS == 0,
	Dn = No; NS = Bn*Dn;
	fprintf('Noise Density to Signal Ratio = %g /Hz',No)
	fprintf(fid,'Noise Density to Signal Ratio = %g /Hz',No);
else
	Dn = NS/Bn;
 	fprintf('Signal/Noise in Bno = %g',1/NS)
 	fprintf(fid,'Signal/Noise in Bno = %g',1/NS);
end
%fprintf('\nOffset %g = asin(%g rad)',Offset,asin(Offset))
fprintf('\n%g degrees modulation at %g rad/sec.',PkDelPhi,SmpRate/MPeriod)
fprintf(fid,'\n%g degrees modulation at %g rad/sec.',PkDelPhi,SmpRate/MPeriod);
Kn = sqrt(Dn*SmpRate/2); 

last = 1 + ncmpo; % index of last output
ending = OutInc*ncmpo; % End time
SmpPerOut = SmpRate*OutInc; % samples per output period
t = linspace(0,ending,last); % linear sequence of times for plots
dt = (t(2) - t(1))/SmpPerOut; % sample time increment
if gop && (gop < 5)
	clf; % clear previous graphs; for v.4, clf #########
end

PkDelRad = PkDelPhi*pi/180;
if NS < 0.03 % less than 2% error from this approximation or from 14 terms
    sig2 = NS;
else
    sig2 = (pi^2)/3;
    for kk = 1:14
        sig2 = sig2 +4*((-1)^kk/kk^2)*besseli(kk,1/NS)/besseli(0,1/NS);
    end % for
end % if
sig2 = sig2/cos(PkDelRad);
fprintf('  Variance in Bno is So2 = %g rad^2',NS)
fprintf('\nTheoretical (FP), 1st order loop, no offset, same So2 in Bno: %g rad^2\n',sig2)
fprintf(fid,'  Variance in Bno is So2 = %g rad^2',NS);
fprintf(fid,'\nTheoretical (FP), 1st order loop, no offset, same So2 in Bno: %g rad^2\n',sig2);

% To help judge inaccuracy of ignoring sampling,
%   compute extra phase shift produced at unity gain
SampPhShift = -(90/pi)*WL*OutInc/SmpPerOut; % radians lag from sampling at WL = K
fprintf('Added phase at unity O.L. gain due to sampling = %g deg.\n',SampPhShift)
fprintf(fid,'Added phase at unity O.L. gain due to sampling = %g deg.\n',SampPhShift);
if truncatePh
	fprintf('Phase will be shown in range 0.5 cycle, modulo 1 cycle.\n')	
	fprintf(fid,'Phase will be shown in range 0.5 cycle, modulo 1 cycle.\n');
else
	fprintf('Phase range will be unrestricted.\n')
	fprintf(fid,'Phase range will be unrestricted.\n');
end

if (NS ~= 0)
    Nstats = 1;
else
    Nstats = 0;
end % conditions OK for noise statistics with known zero mean
if ~Nstats
    fprintf('\nVariances not computed because of zero noise.\n')
    fprintf(fid,'\nVariances not computed because of zero noise.\n');
else
    fprintf('\n')
    fprintf(fid,'\n');
end

%    BEGIN SIMULATION
NL = sin(-Phinit); % <<<<<<<<<<<< THE NON-LINEARITY
% state variables to give specified initial conditions
% X2 and X3 respectively in Eq. (6.50) and (6.51) with D(3) = 0:
x0(1) = (Winit-N(2)*(NL+Offset))/(N(3)-D(2)*N(2));
x0(2) = (Phinit-N(2)*x0(1))/N(3);
e = -Phinit; % initial phase error
freq(1) = Winit; % initial frequency at output
Nplus = 0; Nminus = 0; % skipped cycles
Fsum(1) = 0; Fnet(1) = 0; % rate of skipping

if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
	if e>0
		e = rem(e+pi,2*pi) - pi;
	else
		e = rem(e-pi,2*pi) + pi;
	end
end % if truncatePh
ersum = 0; ersum2 = 0;
msv(1) = NS; % start plot at linear value (avoid problem with ratio in dB)
if (sop ~= 2) 
	er(1) = e;
end
[Phi, Gamma] = c2d(a,b,dt); % values for use in solution eqution [see manual]

running = last; i = 1;
while running
    if (running > 10)% create arrays of required size to speed processing
        er(last)=0;
        PDnoise(last)=0;
        freq(last)=0;
        t(last)=0;
        msv(last)=0;
        if (sop ~= 1)%if following arrays are used (otherwise save memory)
            Fnet(last)=0;
            Fsum(last)=0;
            mnn(last)=0;
        end
        if (gop == 5) && (sop ~= 1)%if following arrays are used
            tshrt(last)=0;
            varshrt(last)=0;
        end
    end % preassigning arrays
    ModBias = (MPeriod-1)/2;
	while i < last, % compute loop responses
		i = i + 1;
        Tsamples = (i-2)*SmpPerOut; % total samples used to here
        ModStart = Tsamples -.9 + round(MPeriod/4); % start modulation at 1/4 period
		for j = 1:SmpPerOut, % loop through unrecorded samples
			DelRad = PkDelRad * sign(ModBias - rem(j+ModStart,MPeriod));
			NL = sin(DelRad - y); % <<<<<<<<<<<< THE NON-LINEARITY 
			if Sources == 2,
                Noise1 = randn; Noise2 = randn;
                PDn = Kn*(Noise1*sin(y)+Noise2*cos(y));
            else
                PDn = Kn*randn;
            end
			u1(1) = NL + Offset + PDn; % initial input to sampler
			u1(2) = u1(1); %  constant Hold output -> same at beginning and end of dt
			x = ltitr(Phi,Gamma,u1',x0); % solutions at period start and end [see manual] 
			yy = c*x'; % corresponding outputs (d is null for polynominals above)
			y = yy(2); % output at end of period is needed
			x0 = x(2,:)'; % final condition gives initial condition for next step
 %           if (i>900) & (i<903)
 %               query = [i,j,DelRad,rem(j+ModStart,MPeriod)]
 %           end			
            eu = -y/(2*pi);
			if eu - Nplus + Nminus > 1
				Nplus = fix(eu + Nminus);
			elseif eu - Nplus + Nminus < -1
				Nminus = -fix(eu - Nplus);
			end
		end % for j			
        PDnoise(i) = PDn;
		e = -y; % untruncated error
		if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
			if e>0
				e = rem(e+pi,2*pi) - pi;
			else
				e = rem(e-pi,2*pi) + pi;
			end
		end % if truncatePh
		if (sop ~= 2)
			er(i) = e + DelRad;
		end
		if (sop == 2)||(sop == 3)
			ersum = ersum + e;
			ersum2 = ersum2 + e^2;
			msv(i) = ersum2/(i);
			mnn(i) = ersum/(i);

			Fsum(i) = (Nplus + Nminus)/(dt*(i-1)*SmpPerOut);
			Fnet(i) = (Nplus - Nminus)/(dt*(i-1)*SmpPerOut);
		end
		% corresponding frequency out from Eq. (6.43) with d3 = 0
		freq(i) = N(2)*u1(2) + x0(1)*(N(3)-N(2)*D(2));        
	end % for i
	
	% END SIMULATION.  The rest is plotting.
	
	clf;
    ncmpo = last - 1;
	ending = ncmpo*OutInc;
	t = linspace(0,ending,last); % reestablish time vector

%****** Start Response Plots for Diagnostics Use	
	if Punits == 1,
		Pscale = 1/(2*pi);
	elseif Punits == 2,
		Pscale = 180/pi;
    else
		Pscale = 1;
	end % if Punits
	
	if Funits == 1,
		Fscale = 1/(2*pi);
	else
		Fscale = 1;
	end % if Funits
	
	if (gop > 0)&&(gop < 5)
		if (gop == 1)||(gop == 2)||(gop == 4), % selecting phase-plane plot
			if gop==1,
				subplot(111),
			elseif gop==2,
				subplot(211),
			elseif gop==4,
				subplot(221),
			end % if gop==1
			plot(Pscale*(er),Fscale*freq,'r')
			title('Phase Plane, Nonlinear Simulation')
			grid
			if Punits == 1, % Set display of phase units
				xlabel('phase error (cycles)');
			elseif Punits == 2,
				xlabel('phase error (degrees)');
            else
				xlabel('phase error (radians)')
			end % if Punits
			if Funits == 1, % Set display of frequency units
				ylabel('freq. out (Hz)');
            else
			 ylabel('freq. out (rad/sec)');
			end % if Funits
		end % if gop ~= 0 or 3
		
		if (gop == 2)||(gop == 4), % selecting phase error vs. time
			if gop == 2,
				subplot (212),
            else
				subplot (223),
			end % if gop==2
			plot(t,Pscale*er,'g')
			title('Phase Error')
			grid
			xlabel('seconds')
			if Punits == 1, % Set display of phase units
				ylabel('cycles');
			elseif Punits == 2,
				ylabel('degrees');
            else
				ylabel('radians');
			end % if Punits
		end % if ()|()
		
		if (gop == 3)||(gop == 4), 
			if gop == 3, % selecting frequency vs. time
				subplot(211);
            else
				subplot(222),
			end % if gop==3 
			plot(t,Fscale*freq,'b')
			title('Frequency Out')
			grid
			xlabel('seconds');
			if Funits == 1, % Set display of frequency units
				ylabel('Hz');
            else
			 ylabel('radians / second');
			end % if Funits
		
			if gop == 3, % selecting Sampler Output vs. time
				subplot(212);
            else
				subplot(224),
			end % if gop==3 
			plot(t,sin(er)+PDnoise+Offset,'+')
			title('Sampler Output')
			grid
			xlabel('seconds')
		end % if ()|()
	elseif (gop == 5)&&((sop == 2)||(sop == 3))
%****** Start Statistical Plots
		tshrt = t(strtplt:last); varshrt = msv(strtplt:last);
		if Nstats
            subplot(221);
            plot([t(strtplt) t(last)],[NS NS],'g',[t(strtplt) t(last)],[sig2 sig2],'m',tshrt,varshrt,'r')
            title('mean square phase (rad^2)')
            grid
            legend('linear','simple (FP)','nonlinear',0)
            subplot(223);
            plot(tshrt,10*log10(varshrt/NS),'b')
            title('rms phase ratio (dB)')
            grid
		end % zero initial conditions
		subplot(222);
		plot(tshrt,mnn(strtplt:last),'k')
		title('mean phase error (rad)')
		grid
		subplot(224);
		plot(tshrt,Fnet(strtplt:last),'g',tshrt,Fsum(strtplt:last),'r')
		title('skips/sec., total and net')
		grid
		xlabel('seconds')
	end % plotting
	
	if (sop == 1)||(sop == 3) % compute from array
        fprintf('\nEstimate from error array:')
        fprintf(fid,'\nEstimate from error array:');
       if Nstats
            arvar = std(er)^2;
            logarvar = 10*log10(arvar/NS);
            fprintf('\n  output variance %g rad^2\n  ratio to So2 %g dB',arvar,logarvar);
            fprintf(fid,'\n  output variance %g rad^2\n  ratio to So2 %g dB',arvar,logarvar);
        end % if Nstats
        armnn = mean(er);
        fprintf('\n  mean phase error %g rad\n  %g points including initial phase\n',armnn,last);
        fprintf(fid,'\n  mean phase error %g rad\n  %g points including initial phase\n',armnn,last);
	end % for error array stats
	if (sop == 2)||(sop == 3) % compute from running stats
       fprintf('Mean square phase error is variance if mean is known to be zero:')
       fprintf(fid,'Mean square phase error is variance if mean is known to be zero:');
        if Nstats
            variance = (msv(last)-(mnn(last))^2)*last/(last-1);
            logvar = 10*log10(variance/NS);
            fprintf('\n  mean square phase %g rad^2, phase variance %g rad^2',msv(last),variance);
            fprintf('\n  variance ratio to So2 %g dB',logvar);
            fprintf(fid,'\n  mean square phase %g rad^2, phase variance %g rad^2',msv(last),variance);
            fprintf(fid,'\n  variance ratio to So2 %g dB',logvar);
        end
        fprintf('\n  %g skipped cycles with %g net skips',Nplus+Nminus,Nplus-Nminus);
		fprintf('\n  mean phase error %g rad\n  %g points (%g seconds) including initial phase\n',mnn(last),last,last*OutInc);	
        fprintf(fid,'\n  %g skipped cycles with %g net skips',Nplus+Nminus,Nplus-Nminus);
		fprintf(fid,'\n  mean phase error %g rad\n  %g points (%g seconds) including initial phase\n',mnn(last),last,last*OutInc);	
    end % for running stats
	
	sound(sin(6283*[0:1/8192:0.2]),8192); %0.2 seconds of 1 kHz tone;
	running = 0;
    pause(1)
end % while running
fclose(fid);
end % for variable 2
end % for variable 1