% Mean Time to First Skip in Noise, Second-Order Loop
%   SINE NONLINEARITY
% [24 seconds for 10,000 increments at SmpPerWL=100, sop=2, Athlon 3000+]
% Tollboxes Required: control
clear;
for alpha = [.985] %variable 1
for z = [1] %variable 2
for NS = [2 1 .5 .3333 .25]% %variable 3
clear Favg FbarOver Fnet Fsum N Ndif Nsum PDnoise Tavg Tshrt Ttime freq mnn t var varshrt 

% MODIFY PARAMETERS BELOW & GO
%*****************************
% Set either of the following 2 parameters and set the other to zero.
%NS = .5; % N/S in Loop Bandwidth
No = 0; % Noise Power Density, per Hz, to Signal Ratio
Sources = 2; % 1 or 2 independent noise sources
Phinit = 0;  % radian initial phase
Winit = 0; % radian/sec initial frequency
Wn = 1;      % rad/sec natural frequency
%z = 1.4;    % damping factor
%alpha = .97;   % Eq. (6.5)
SmpPerWL = 100; % ratio of sample rate to unity-gain radian bandwidth 
TshowXWL = 1; % (period between outputs) X (unity-gain radian bandwidth)
ncmpo = 4000; % approximate output points (reduced by skips/2)
Offset = 0;     % offset at PD output (same sign as steady-state output)
		% Choose Graphical Output
truncatePh = 1; % 1:truncate phase; 0:do not
strtplt = 1; % skip first plotted points in statistical plots up to strtplt
	%  sop=2:  Compute running stats (no array)
	%  sop=3:  Compute array and running stats
	sop = 2;  % Set sop = 2 normally.
%******************************	

fprintf('\n\nMEAN TIME TO FIRST CYCLE SKIP (Tm), iterative form, Skip2i (4/12/07)')
fprintf('\n2nd-order loop with SINUSOIDAL Phase Detector Characteristic')
fprintf('\nWn = %g rad/sec; zeta = %g; alpha = %g',Wn,z,alpha)
Bn = Wn*(1/(2*z) + 2*z*alpha^2)/4; % Noise BW in Hz
fprintf('\nNoise Bandwidth (linear), Bno = %g Hz.  Number of noise sources: %u',Bn,Sources)
warning off MATLAB:divideByZero
kOverWn = 1/((1-alpha)*2*z);
fprintf('\nOffset is %g, producing %g rad/sec offset in center frequency\n',Offset,Offset*kOverWn*Wn)
warning on MATLAB:divideByZero

%Polynominals for open loop gain:
	N = [0     2*z*Wn*alpha   Wn^2]; % numerator
	D = [1   2*z*Wn*(1-alpha)   0 ]; % denominator

[a,b,c,d] = tf2ss(N,D); % elements of dynamic matrix equation
y = Phinit; % initial phase at output

%Compute unity-gain bandwidth
factor = N(2)^2-D(2)^2;
WL = sqrt((factor+sqrt(factor^2+4*N(3)^2))/2); % open-loop gain = 1

SmpRate = SmpPerWL*WL; % sample rate
OutInc = TshowXWL/WL;% period between outputs

% Compute multiplier of noise distribution having unity power.
if NS == 0,
	Dn = No;
	fprintf('Noise Density to Signal Ratio = %g /Hz',No)
	NS = Dn*Bn;
else
	Dn = NS/Bn;
 	fprintf('Signal/Noise in Bno = %g',1/NS)
end
rho = 1/NS;
Kn = sqrt(Dn*SmpRate/2); 

last = 1 + ncmpo; % index of last output
ending = OutInc*ncmpo; % End time
SmpPerOut = SmpRate*OutInc; % samples per output perio
dt = 1/SmpRate; % sample time increment

Tm = pi^2*rho*(besseli(0,rho))^2/(2*Bn);
fprintf('\nTheoretical Mean Time to First Skip, 1st-order loop: %g seconds.\n',Tm);
fprintf('                                            1/(TmBno): %g.\n',1/Tm/Bn);

% To help judge inaccuracy of ignoring sampling,
%   compute extra phase shift produced at unity gain
SampPhShift = -(90/pi)*WL*OutInc/SmpPerOut; % radians lag from sampling at unity open-loop gain
fprintf('Added phase at unity O.L. gain due to sampling = %g deg.\n',SampPhShift)
fprintf('Phase will be reset to 0 and skip counted when outside 1 cycle.\n')	

dt = 1/SmpRate;
Nplus = 0; Nminus = 0; % skipped cycles
Tavg(1) = 0; % rate of skipping
Favg(1) = 0; % reciprocal of Tavg
Nrel(1) = 0; % relative differential in skips
time = 0; % time at last skip
%    BEGIN SIMULATION
running = last; i = 1; ii = 1; Nplus = 0; Nminus = 0; Tskips = 0;
Nsum(1) = 0; Ndif(1) = 0;	
er(1) = -Phinit; % initial phase error
freq(1) = Winit; % initial frequency at output
while running	
    if (running > 10)
        Favg(last)=0;
        FbarOver(last)=0;
        Fnet(last)=0;
        Fsum(last)=0;
        N(last)=0;
        Ndif(last)=0;
        Nsum(last)=0;
        PDnoise(last)=0;
        Tavg(last)=0;
        Tshrt(last)=0;
        Ttime(last)=0;
        freq(last)=0;
        mnn(last)=0;
        t(last)=0;
        var(last)=0;
        varshrt(last)=0;
    end % preassigning arrays
	while i < last
		NL = sin(-Phinit); % <<<<<<<<<<<< THE NON-LINEARITY
		% state variables to give specified initial conditions
		% X2 and X3 respectively in Eq. (6.50) and (6.51) with D(3) = 0:
		x0(1) = (Winit-N(2)*(NL+Offset))/(N(3)-D(2)*N(2));
		x0(2) = (Phinit-N(2)*x0(1))/N(3);
		[Phi, Gamma] = c2d(a,b,dt); % values for use in solution eqution [see manual]
		StartOver = 0; k = -1;
		while ~StartOver && (i < last), % compute loop responses
			i = i + 1; k = k+1;
			for j = 1:SmpPerOut, % loop through unrecorded samples
				NL = sin(-y); % <<<<<<<<<<<< THE NON-LINEARITY 
                if Sources == 2,
                    Noise1 = randn; Noise2 = randn;
                    PDn = Kn*(Noise1*sin(y)+Noise2*cos(y));
                else
                    PDn = Kn*randn;
                end	
				u1(1) = NL + Offset + PDn; % initial input to sampler
				u1(2) = u1(1); %  constant Hold output -> same at beginning and end of dt
				x = ltitr(Phi,Gamma,u1',x0); % solutions at period start and end [see manual] 
				yy = c*x'; % corresponding outputs (d is null for polynominals above)
				y = yy(2); % output at end of period is needed
				x0 = x(2,:)'; % final condition gives initial condition for next step
				if y<-2*pi
					StartOver = 1;
					Nplus = 1 + Nplus;
				elseif y>2*pi
					StartOver = 1;
					Nminus = Nminus + 1;
				end	
				if StartOver
					time = dt*(j - .5 + k*SmpPerOut) + time;
					break;
				end
			end % for j
			if StartOver
				ii = ii + 1;
				Nsumm = Nplus + Nminus;
				Nsum(ii) = Nsumm;
				Ndif(ii) = Nplus - Nminus;
				if Nplus||Nminus
					Tavg(ii) = time/Nsumm;
					Favg(ii) = Nsumm/time;
				else
					Tavg(ii) = 0;
					Favg(ii) = 0;
				end
				Ttime(ii) = time;
%				eyehi(ii) = i-1;  % TEST TEST TEST TEST TEST TEST
%				eyelo(ii) = i-1-ii;  % TEST TEST TEST TEST TEST TEST
			end
			e = -y; % untruncated error
			if (sop ~= 2)
				er(i) = e;
				freq(i) = N(2)*u1(2) + x0(1)*(N(3)-N(2)*D(2)); % corresponding frequency out from Eq. (6.43) with d3 = 0
				PDnoise(i) = PDn; 
			end 
		end % while ~StartOver
	end % while i < last
	% END SIMULATION.  The rest is plotting.
	
	ncmpo = last - 1;
	ending = ncmpo*OutInc;
    clf; % clear graphs
    
    %****** Start Mean Time Plots
    if (strtplt < ii);
        Tshrt = Ttime(strtplt:ii);
        FbarOver = Favg/Bn;
        FOTheory = 1/(Bn*Tm);
        
        subplot(211);
        plot([Tshrt(1) Ttime(ii)],[FOTheory FOTheory],'g',Tshrt,FbarOver(strtplt:ii),'r')
        title(['1/(TmBno) for S/N =',num2str(rho)]);
        grid
        xlabel('Simulated Time (seconds)')
        legend('theoretical for 1st-order','simulated 2nd-order',0)
        
        subplot(212);
        plot(Tshrt,Ndif(strtplt:ii),'om',Tshrt,Nsum(strtplt:ii),'+b')
        title('Total and Differential Skips')
        grid
        xlabel('Simulated Time (seconds)')
        
    end % plotting
    
    T21 = Tavg(ii);
    fprintf('\n  Average time to first skip is %g seconds, 1/(TmBo) = %g',T21,1/T21/Bn);
	fprintf('\n  %g total skips with plus exceeding minus by %g',Nsum(ii),Ndif(ii));
	fprintf('\n  %g points including initial phase',last);
	sound(sin(6283*[0:1/8192:0.2]),8192); %0.2 seconds of 1 kHz tone
	running = 0;
    pause(1)
end % while running
end %  %variable 3
end %  %variable 2
end %  %variable 1